# -*- coding: utf-8 -*-
# _synthesizeNTF.py
# Module providing the synthesizeNTF function
# Copyright 2013 Giuseppe Venturini
# This file is distributed with python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.
#
# The following code has been (slightly) modified from pydsm, its original
# copyright notice follows:
#
# Copyright (c) 2012, Sergio Callegari
# All rights reserved.
#
# The py dsm code was ported from the MATLAB Delta Sigma toolbox
# Copyright (c) 2009, Richard Schreier
#
# The three software follow the same license, known as the 2-clause BSD.
# See the LICENSE file for details.

"""
Module providing the main NTF synthesis function.
"""

from warnings import warn

import numpy as np

from ._config import optimize_NTF
from ._synthesizeNTF0 import synthesizeNTF0
from ._synthesizeNTF1 import synthesizeNTF1


def synthesizeNTF(order=3, osr=64, opt=0, H_inf=1.5, f0=0.0):
    """Synthesize a noise transfer function for a delta-sigma modulator.

    **Parameters:**

    order : int, optional
        the order of the modulator, defaults to 3
    osr : float, optional
        the oversamping ratio, defaults to 64
    opt : int or list of floats, optional
        flag for optimized zeros, defaults to 0

        * 0 -> not optimized,
        * 1 -> optimized,
        * 2 -> optimized with at least one zero at band-center,
        * 3 -> optimized zeros (with optimizer)
        * 4 -> same as 3, but with at least one zero at band-center
        * [z] -> zero locations in complex form

    H_inf : real, optional
        max allowed peak value of the NTF. Defaults to 1.5
    f0 : real, optional
        center frequency for BP modulators, or 0 for LP modulators.
        Defaults to 0.
        1 corresponds to the sampling frequency, so that 0.5 is the
        maximum value. Value 0 specifies an LP modulator.

    **Returns:**

    ntf : tuple
        noise transfer function in zpk form.

    **Raises:**

    ValueError
        * 'Error. f0 must be less than 0.5' if f0 is out of range

        * 'Order must be even for a bandpass modulator.' if the order is
          incompatible with the modulator type.

        * 'The opt vector must be of length xxx' if opt is used to explicitly
          pass the NTF zeros and these are in the wrong number.

    **Warns:**

        * 'Creating a lowpass ntf.' if the center frequency is different
          from zero, but so low that a low pass modulator must be designed.

        * 'Unable to achieve specified H_inf ...' if the desired H_inf
          cannot be achieved.

        * 'Iteration limit exceeded' if the routine converges too slowly.

    **Notes:**

    This is actually a wrapper function which calls the appropriate version
    of synthesizeNTF, based on the module control flag `optimize_NTF` which
    determines whether to use optimization tools.

    Parameter ``H_inf`` is used to enforce the Lee stability criterion.

    **Example:**

    Fift-order lowpass modulator; zeros optimized for an oversampling ratio of 32.::

        from deltasigma import *
        H = synthesizeNTF(5, 32, 1)
        pretty_lti(H)

    Returns::

              (z -1) (z^2 -1.997z +1) (z^2 -1.992z +0.9999)
        --------------------------------------------------------
         (z -0.7778) (z^2 -1.796z +0.8549) (z^2 -1.613z +0.665)

    .. plot::

        from deltasigma import *
        H = synthesizeNTF(5, 32, 1)
        DocumentNTF(H, 32)

    .. seealso::

       :func:`clans` : Closed-Loop Analysis of Noise-Shaper.
               An alternative method for selecting NTFs based on the 1-norm of the
               impulse response of the NTF

       :func:`synthesizeChebyshevNTF()` : Select a type-2 highpass Chebyshev NTF.
           This function does a better job than synthesizeNTF if osr
           or H_inf is low.

    """
    if f0 > 0.5:
        raise ValueError('Error. f0 must be less than 0.5.')
    if f0 != 0 and f0 < 0.25/osr:
        warn('Creating a lowpass ntf.')
        f0 = 0
    if f0 != 0 and order % 2 != 0:
        raise ValueError('Order must be even for a bandpass modulator.')
    opt = np.asarray(opt)
    if opt.ndim > 1 or (opt.ndim == 1 and opt.size != order):
        raise ValueError('The opt vector must be of length %d.' % order)

    if not optimize_NTF:
        ntf = synthesizeNTF0(order, osr, opt, H_inf, f0)
    else:
        ntf = synthesizeNTF1(order, osr, opt, H_inf, f0)
    return ntf



# -*- coding: utf-8 -*-
# _bquantize.py
# Bipolar quantization module
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the bquantize() function, used to bidirectionally 
quantize a vector to signed digits.
"""

from __future__ import division, print_function

import numpy as np

from ._constants import eps
from ._utils import empty, mfloor


def bquantize(x, nsd=3, abstol=eps, reltol=10 * eps):
    """Bidirectionally quantize a 1D vector ``x`` to ``nsd`` signed digits.

    This method will terminate early if the error is less than the specified
    tolerances.

    The quantizer details are repeated here for the user's convenience:

        The quantizer is ideal, producing integer outputs centered about zero.
        Quantizers with an even number of levels are of the mid-rise type and
        produce outputs which are odd integers. Quantizers with an odd number
        of levels are of the mid-tread type and produce outputs which are even
        integers.

        .. image:: ../doc/_static/quantizer_model.png
            :align: center
            :alt: Quantizer model

    **Parameters:**

    x : array_like or sequence
           the data to be quantized.

    nsd : int, optional
          The number of signed digits.

    abstol and reltol : floats, optional
        If not supplied, the absolute tolerance and the relative 
        tolerance default to ``eps`` and ``10*eps``, resp.

    **Returns:**

    y : list 
        List of objects described below.

    ``y`` is a list of instances with the same length as ``x`` and the 
    following attributes:

    * ``y[i].val`` is the quantized value in floating-point form,
    * ``y[i].csd`` is a 2-by-nsd (or less) matrix containing
      the powers of two (first row) and their signs (second row).
    
    .. seealso:: 
        :func:`bunquantize`, :func:`ds_quantize`

    """

    n = x.shape[0] if isinstance(x, np.ndarray) else len(x)
    #q = np.zeros((2*n, nsd)) in the original source #rep?
    y = [empty() for i in range(n)]
    offset = -np.log2(0.75)

    for i in range(n):
        xp = x[i]
        y[i].val = 0.
        y[i].csd = np.zeros((2, 0), dtype='int16')
        for _ in range(nsd):
            error = np.abs(y[i].val - x[i])
            if error <= abstol and error <= np.abs(x[i]) * reltol:  # rep? in the orig: or
                break
            p = mfloor(np.log2(np.abs(xp)) + offset)
            p2 = 2 ** p
            sx = np.sign(xp)
            xp = xp - sx * p2
            y[i].val = y[i].val + sx * p2
            addme = np.array((p, sx)).reshape((2, 1))
            y[i].csd = np.concatenate((y[i].csd, addme), axis=1)
    return y



# -*- coding: utf-8 -*-
# test_undbm.py
# This module provides the tests for the undbm function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the undbm() function.
"""

import unittest
import numpy as np

from deltasigma import undbm

class TestUndbm(unittest.TestCase):
    """Test class for undbm()"""

    def setUp(self):
        pass

    def test_undbm_1(self):
        """Test function for undbm() 1/3"""
        self.assertTrue(np.allclose([undbm(53.015)], [100.054125892], rtol=1e-05,
                                    atol=1e-08))

    def test_undbm_2(self):
        """Test function for undbm() 2/3"""
        self.assertTrue(np.allclose([undbm(3, 100)], [0.44668359215], rtol=1e-05,
                                    atol=1e-08))

    def test_undbm_3(self):
        """Test function for undbm() 3/3"""
        self.assertTrue(np.isscalar(undbm(3, 100)))




# -*- coding: utf-8 -*-
# test_calculateTF.py
# This module provides the tests for the calculateTF() function.
# Copyright 2014 Giuseppe Venturini & Shayne Hodge
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

import unittest
import numpy as np
import deltasigma as ds
from deltasigma._utils import cplxpair


class TestCalculateTF(unittest.TestCase):
    """Test function for calculateTF()"""
    def setUp(self):
        ABCD = [[1.0, 0.0, 0.0, 0.044408783846879, -0.044408783846879],
                [0.999036450096481, 0.997109907515262, -0.005777399147297,
                 0.0, 0.499759089304780],
                [0.499759089304780, 0.999036450096481, 0.997109907515262,
                 0.0, -0.260002096136488],
                [0.0, 0.0, 1.0,  0.0, -0.796730400347216]]
        ABCD = np.array(ABCD)
        (ntf, stf) = ds.calculateTF(ABCD)
        (ntf_zeros, ntf_poles) = (np.roots(ntf.num), np.roots(ntf.den))
        (stf_zeros, stf_poles) = (np.roots(stf.num), np.roots(stf.den))
        mntf_poles = np.array((1.498975311463384, 1.102565142679772,
                               0.132677264750882))
        mntf_zeros = np.array((0.997109907515262 + 0.075972576202904j,
                               0.997109907515262 - 0.075972576202904j,
                               1.000000000000000 + 0.000000000000000j))
        mstf_zeros = np.array((-0.999999999999996,))
        mstf_poles = np.array((1.498975311463384, 1.102565142679772,
                               0.132677264750882))

        # for some reason, sometimes the zeros are in different order.
        (self.ntf_zeros, self.mntf_zeros) = (cplxpair(ntf_zeros),
                                             cplxpair(mntf_zeros))
        (self.stf_zeros, self.mstf_zeros) = (cplxpair(stf_zeros),
                                             cplxpair(mstf_zeros))
        (self.ntf_poles, self.mntf_poles) = (cplxpair(ntf_poles),
                                             cplxpair(mntf_poles))
        (self.stf_poles, self.mstf_poles) = (cplxpair(stf_poles),
                                             cplxpair(mstf_poles))

    def test_ntf_zeros_equal(self):
        self.assertTrue(
            np.allclose(self.ntf_zeros, self.mntf_zeros, rtol=1e-5, atol=1e-8))

    def test_ntf_poles_equal(self):
        self.assertTrue(
            np.allclose(self.ntf_poles, self.mntf_poles, rtol=1e-5, atol=1e-8))

    def test_stf_zeros_equal(self):
        self.assertTrue(
            np.allclose(self.stf_zeros, self.mstf_zeros, rtol=1e-5, atol=1e-8))

    def test_stf_poles_equal(self):
        self.assertTrue(
            np.allclose(self.stf_poles, self.mstf_poles, rtol=1e-5, atol=1e-8))



# -*- coding: utf-8 -*-
# test_ds_freq.py
# This module provides the tests for the ds_freq() function.
# Copyright 2014 Giuseppe Venturini & Shayne Hodge
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

import unittest
import numpy as np
import deltasigma as ds


class TestDSFreq(unittest.TestCase):
    """Tests ds_freq()"""

    def setUp(self):
        pass

    def test_res_equals_tres(self):
        """Check ds_freq() output to known values."""
        a = ds.ds_freq(osr=128, f0=0.0, quadrature=True)
        b = np.diff(a)
        res = (0.00190595677588, 0.00510204081633,
               0.207803148686, 0.00491921819577)
        tres = (a.mean(), b.mean(), a.std(), b.std())
        self.assertTrue(np.allclose(res, tres, atol=1e-8, rtol=1e-5))



# -*- coding: utf-8 -*-
# _pulse.py
# This module provides the pulse function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the pulse() function, which calculates the sampled
pulse response of a CT system.
"""

from __future__ import division

import collections

import numpy as np
from scipy.signal import lti, step2

from ._utils import _is_A_B_C_D, _is_num_den, _is_zpk, lcm, rat


def pulse(S, tp=(0., 1.), dt=1., tfinal=10., nosum=False):
    """Calculate the sampled pulse response of a CT system.

    ``tp`` may be an array of pulse timings, one for each input, or even a
    simple 2-elements tuple.

    **Parameters:**

    S : sequence
        A sequence of LTI objects specifying the system.

    The sequence S should be assembled so that ``S[i][j]`` returns the
    LTI system description from input ``i`` to the output ``j``.

    In the case of a MISO system, a unidimensional sequence ``S[i]``
    is also acceptable.

    tp : array-like
        An (n, 2) array of pulse timings

    dt : scalar
        The time increment

    tfinal : scalar
        The time of the last desired sample

    nosum : bool
        A flag indicating that the responses are not to be summed

    **Returns:**

    y : ndarray
        The pulse response
    """
    tp = np.asarray(tp)
    if len(tp.shape) == 1:
        if not tp.shape[0] == 2:
            raise ValueError("tp is not (n, 2)-shaped")
        tp = tp.reshape((1, tp.shape[0]))
    if len(tp.shape) == 2:
        if not tp.shape[1] == 2:
            raise ValueError("tp is not (n, 2)-shaped")

    # Compute the time increment
    dd = 1
    for tpi in np.nditer(tp.T.copy(order='C')):
        _, di = rat(tpi, 1e-3)
        dd = lcm(di, dd)

    _, ddt = rat(dt, 1e-3)
    _, df = rat(tfinal, 1e-3)
    delta_t = 1./lcm(dd, lcm(ddt, df))
    delta_t = max(1e-3, delta_t)    # Put a lower limit on delta_t
    if (isinstance(S, collections.Iterable) and len(S)) \
       and (isinstance(S[0], collections.Iterable) and len(S[0])) \
           and (isinstance(S[0][0], lti) or _is_zpk(S[0][0]) or _is_num_den(S[0][0]) \
                or _is_A_B_C_D(S[0][0])):
        pass
    else:
        S = list(zip(S)) #S[input][output]
    y1 = None
    for Si in S:
        y2 = None
        for So in Si:
            _, y2i = step2(So, T=np.arange(0., tfinal + delta_t, delta_t))
            if y2 is None:
                y2 = y2i.reshape((y2i.shape[0], 1, 1))
            else:
                y2 = np.concatenate((y2,
                                     y2i.reshape((y2i.shape[0], 1, 1))),
                                    axis=1)
        if y1 is None:
            y1 = y2
        else:
            y1 = np.concatenate((y1, y2), axis=2)

    nd = int(np.round(dt/delta_t, 0))
    nf = int(np.round(tfinal/delta_t, 0))
    ndac = tp.shape[0]

    ni = len(S) # number of inputs

    if ni % ndac != 0:
        raise ValueError('The number of inputs must be divisible by the number of dac timings.')
        # Original comment from the MATLAB sources:
        # This requirement comes from the complex case, where the number of inputs
        # is 2 times the number of dac timings. I think this could be tidied up.

    # nis: Number of inputs grouped together with a common DAC timing
    # (2 for the complex case)
    nis = int(ni/ndac)

    # notice len(S[0]) is the number of outputs for us
    if not nosum: # Sum the responses due to each input set
        y = np.zeros((np.ceil(tfinal/float(dt)) + 1, len(S[0]), nis))
    else:
        y = np.zeros((np.ceil(tfinal/float(dt)) + 1, len(S[0]), ni))

    for i in range(ndac):
        n1 = int(np.round(tp[i, 0]/delta_t, 0))
        n2 = int(np.round(tp[i, 1]/delta_t, 0))
        z1 = (n1, y1.shape[1], nis)
        z2 = (n2, y1.shape[1], nis)
        yy = + np.concatenate((np.zeros(z1), y1[:nf-n1+1, :, i*nis:(i + 1)*nis]), axis=0) \
             - np.concatenate((np.zeros(z2), y1[:nf-n2+1, :, i*nis:(i + 1)*nis]), axis=0)
        yy = yy[::nd, :, :]
        if not nosum: # Sum the responses due to each input set
            y = y + yy
        else:
            y[:, :, i] = yy.reshape(yy.shape[0:2])
    return y



# -*- coding: utf-8 -*-
# test_dbv.py
# This module provides the tests for the dbv function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the dbv() function.
"""

import unittest
import numpy as np

from deltasigma import dbv, undbv

class TestDbv(unittest.TestCase):
    """Test class for dbv()"""
    def setUp(self):
        self.t1 = np.array([3.0])
        self.r1 = 9.5424250943932485

    def test_dbv(self):
        """Test function for dbv()"""
        res = dbv(self.t1)
        t2 = undbv(res)
        self.assertTrue(np.allclose(t2, self.t1, atol=1e-8, rtol=1e-5))
        self.assertTrue(np.allclose(res, self.r1, atol=1e-8, rtol=1e-5))




# -*- coding: utf-8 -*-
# _thermometer.py
# This module provides the thermometer function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the thermometer() function.
"""

import numpy as np


def thermometer(x, m):
    """Convert x to thermometer (aka unary) code

    **Parameters:**

    x : 1-D ndarray
        The array of positive ints, each of which will be converted.

    m : int
        total length of the thermometer array.

    **Returns:**

    t : ndarray
        ``t`` is an m by ``len(x)`` matrix wherein the first
        ``x(i)`` components of column ``i`` are one.
    """
    t = np.zeros((m, len(x)))
    for i in range(len(x)):
        t[:x[i], i] = np.ones((x[i], ))
    return t



# -*- coding: utf-8 -*-
# _DocumentNTF.py
# Module providing DocumentNTF
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the DocumentNTF function.
"""

from __future__ import division

import numpy as np
import pylab as plt

from ._calculateTF import calculateTF
from ._dbv import dbv
from ._ds_f1f2 import ds_f1f2
from ._ds_freq import ds_freq
from ._evalTF import evalTF
from ._figureMagic import figureMagic
from ._infnorm import infnorm
from ._plotPZ import plotPZ
from ._rmsGain import rmsGain


def DocumentNTF(arg1, osr=64, f0=0, quadrature=False):
    """Plot the NTF's poles and zeros as well as its frequency-response

    The first argument is either the NTF or ABCD matrix.
    If the first argument is ABCD, the STF is also plotted.

    .. plot::

       from deltasigma import *
       import numpy as np
       import pylab as plt
       order = 4
       osr = 64
       nlev = 2
       f0 = 0.
       Hinf = 1.5
       form = 'CRFB'
       ntf = synthesizeNTF(order, osr, 2, Hinf, f0)
       a, g, b, c = realizeNTF(ntf, form)
       b = np.concatenate(( # Use a single feed-in for the input
                           np.atleast_1d(b[0]),
                           np.zeros((b.shape[0] - 1,))
                         ))
       ABCD = stuffABCD(a, g, b, c, form)
       DocumentNTF(ABCD, osr, f0)

    """
    if isinstance(arg1, np.ndarray):
        ABCD = arg1
        ntf, stf = calculateTF(ABCD)
    else:
        ntf = arg1
        stf = None

    fig = plt.figure(figsize=(12, 5))
    plt.subplot(121)
    plotPZ(ntf, 'b', 6, showlist=False)
    plt.title('Poles and Zeros')
    plt.subplot(122)
    f = ds_freq(osr, f0, quadrature)
    z = np.exp(2j * np.pi * f)
    H = dbv(evalTF(ntf, z))
    plt.plot(f, H, 'b')

    if stf is not None:
        fig.suptitle('NTF and STF', fontsize=14)
        G = dbv(evalTF(stf, z))
        plt.hold(True)
        plt.plot(f, G, 'm')
        plt.hold(False)
    else:
        fig.suptitle('NTF', fontsize=14)

    f1, f2 = ds_f1f2(osr, f0, quadrature)
    NG0 = dbv(rmsGain(ntf, f1, f2))
    plt.hold(True)
    plt.plot(np.array([f1, f2]), NG0*np.array([1, 1]), 'k', linewidth=3)

    if f0  ==  0:
        plt.text(0.5/osr, NG0, '  %.0fdB' % NG0, horizontalalignment = 'left',
                 verticalalignment = 'center')
    else:
        plt.text(f0, NG0 + 1, '%.0fdB' % NG0, horizontalalignment = 'center',
                 verticalalignment = 'bottom')
    msg = ' Inf-norm of H = %.2f\n 2-norm of H = %.2f' % (infnorm(ntf)[0], rmsGain(ntf, 0, 1))
    if quadrature:
        ING0 = dbv(rmsGain(ntf, - f1, - f2))
        plt.plot(-np.array([f1, f2]), ING0*np.array([1, 1]), 'k', linewidth=3)
        plt.text(-f0, ING0 + 1, '%.0fdB' % ING0, horizontalalignment = 'center',
                 verticalalignment = 'bottom')
        f_left = - 0.5
    else:
        f_left = 0
    # variable 'f_left' used before assignment in DocumentNTF.m #REP
    if f0 < 0.25:
        plt.text(0.48, 0, msg, horizontalalignment = 'right', verticalalignment = 'top')
    else:
        plt.text(f_left, 0, msg, horizontalalignment = 'left', verticalalignment = 'top')
    plt.grid(True)
    y_bot = min(-80, np.round(NG0*1.1, -1))
    figureMagic(xRange=(f_left, 0.5), dx=1/20., yRange=(y_bot, 15), dy=10)
    plt.ylabel('|H(f)| dB')
    plt.xlabel('Frequency ($1 \\rightarrow f_{s}$)')
    plt.title('Frequency Response')
    return



# -*- coding: utf-8 -*-
# _figureMagic.py
# Module providing figureMagic()
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the utility function figureMagic()
"""

from __future__ import division

from warnings import warn

import numpy as np
import pylab as plt


def figureMagic(xRange=None, dx=None, xLab=None, yRange=None, dy=None,
                yLab=None, size=None, name=None):
    """Utility function to quickly set plot parameters.

    **Parameters:**

    xRange : 2 elements sequence, optional
        set the x-axis limits

    dx : scalar, optional
        set the ticks spacing on the x-axis

    xLab : any, optional
        Ignored variable, only accepted for compatibility with the MATLAB
        syntax.

    yRange : 2 elements sequence, optional
        set the y-axis limits

    dy : scalar, optional
        set the ticks spacing on the y-axis

    yLab : any, optional
        Ignored variable, only accepted for compatibility with the MATLAB
        syntax.

    size : 2-elements sequence, optional
        Figure size, in inches.

    name : string, optional
        Title for the current plot (or subplot)

    All parameters are optional and any unspecified figure parameter is
    left untouched.

    .. plot::

       import numpy as np
       import pylab as plt
       from deltasigma import figureMagic
       t = np.linspace(0, 1)
       a = np.sin(2*np.pi*t + np.pi/4)
       plt.subplot(121)
       plt.plot(t, a)
       plt.title("Before")
       plt.subplot(122)
       plt.plot(t, a)
       plt.title("After")
       figureMagic([0, 1], dx=.1, xLab=None, yRange=[-1.2, 1.2],
                   dy=.2, yLab=None, size=(8, 4), name="After")

    """
    fig = plt.gcf()
    if size is not None and len(size):
        fig.set_size_inches(size)

    plt.grid(True)

    ax = plt.gca()
    if name is not None:
        ax.set_title(name, fontsize=14)

    if xRange is not None or yRange is not None:
        ax.set_autoscale_on(False)
        ax.set_aspect('auto', 'box')

    if xRange is not None:
        ax.set_xlim(xRange)
    if yRange is not None:
        ax.set_ylim(yRange)

    if dx is not None:
        x1, x2 = ax.get_xlim()
        ax.xaxis.set_ticks(np.arange(x1, x2, dx))
    if dy is not None:
        y1, y2 = ax.get_ylim()
        ax.yaxis.set_ticks(np.arange(y1, y2, dy))
    # ax.xaxis.set_major_formatter(ticker.FormatStrFormatter('%0.1f'))

    if xLab is not None:
        warn(('figureMagic() got xLab=%s, but xLab is not implemented and ' +
              'will be ignored.') % xLab)
    if yLab is not None:
        warn(('figureMagic() got yLab=%s, but xLab is not implemented and ' +
              'will be ignored.') % yLab)
    return



# -*- coding: utf-8 -*-
# _clans.py
# Closed-Loop Analysis of Noise-Shapers module
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the clans() optimization function.
"""

from __future__ import division, print_function

import numpy as np
from scipy.optimize import minimize
from scipy.signal import dimpulse

from ._dsclansNTF import dsclansNTF
from ._evalTF import evalTF
from ._synthesizeNTF import synthesizeNTF
from ._utils import cplxpair


# We ported the version that was originally designed for the MATLAB
# Optimization Toolbox version >= 6

def clans(order=4, OSR=64, Q=5, rmax=0.95, opt=0):
    """Optimal NTF design for a multi-bit modulator.
    
    Synthesize a noise transfer function (NTF) for a lowpass delta-sigma 
    modulator using the CLANS methodology.

    CLANS stands for Closed-Loop Analysis of Noise-Shapers,
    and it was originally developed by J.G. Kenney and L.R. Carley [1]_.

    .. [1] J. G. Kenney and L. R. Carley, “Design of multibit
        noise-shaping data converters,” Analog Integrated Circuits Signal
        Processing Journal, vol. 3, pp. 259-272, 1993.

    **Parameters:**

    order : int
        The order of the NTF.

    OSR : int
        The oversampling ratio.

    Q : int
        The maximum number of quantization levels used by the fed-back quantization
        noise. (Mathematically, :math:`Q = \\|h\\|_1 - 1`, i.e. the sum of the 
        absolute values of the impulse response samples minus one is the 
        maximum instantaneous noise gain.)

    rmax : float
        The maximum radius for the NTF poles.

    opt : int
        A flag used to request optimized NTF zeros. 
        
    * `opt=0` puts all NTF zeros at band center (DC for lowpass modulators).
    * `opt=1` optimizes the NTF zeros.
    * For even-order modulators, `opt=2` puts two zeros at band-center, 
      but optimizes the rest.

    **Returns**

    ntf : tuple
        The modulator NTF, given in ZPK (zero-pole-gain) form.

    **Example:**

    Fifth-order lowpass modulator; (time-domain) noise gain of 5, 
    zeros optimized for OSR = 32.::

        H = clans(5, 32, 5, .95, 1)
        pretty_lti(H)

    Returns::

              (z -1) (z^2 -1.997z +1) (z^2 -1.992z +0.9999)      
        ---------------------------------------------------------
         (z -0.4184) (z^2 -1.305z +0.5713) (z^2 -0.978z +0.2686) 

    ``H`` can be plotted through :func:`DocumentNTF`:

    .. plot::

        from deltasigma import DocumentNTF, clans
        # Fifth-order lowpass modulator; (time-domain) noise gain of 5, 
        # zeros optimized for OSR = 32.
        H = clans(5, 32, 5, .95, 1)
        DocumentNTF(H)

    """
    # Create the initial guess
    Hz, poles, _ = synthesizeNTF(order, OSR, opt, 1 + Q, 0)
    x = np.zeros((order, ))
    odd = order % 2
    poles = cplxpair(poles)
    poles = poles[::-1]
    if odd == 1:
        z = poles[0]/rmax
        if (np.abs(z) > 1).any(): #project poles outside rmax onto the circle
            z = z/np.abs(z)
        s = (z - 1)/(z + 1)
        x[0] = np.real_if_close(np.sqrt(-s))
    for i in range(odd, order, 2):
        z = poles[i:i + 2]/rmax
        if np.any(np.abs(z) > 1): #project poles outside rmax onto the circle
            z = z/np.abs(z)
        s = (z - 1)/(z + 1)
        coeffs = np.poly(s)
        wn = np.sqrt(coeffs[2])
        zeta = coeffs[1]/(2 * wn)
        x[i] = np.real_if_close(np.sqrt(zeta))
        x[i + 1] = np.real_if_close(np.sqrt(wn))
    #options=optimset('TolX',1e-06,'TolFun',1e-06,'TolCon',1e-06,'MaxIter',1000)
    #fobj = lambda x: dsclansObj6a(x, order, OSR, Q, rmax, Hz)
    #fconstr = lambda x: dsclansObj6b(x, order, OSR, Q, rmax, Hz)
    res = minimize(dsclansObja, x, args=(order, OSR, Q, rmax, Hz), 
             method='slsqp', constraints={'type':'ineq', 'fun':dsclansObjb, 
             'args':(order, OSR, Q, rmax, Hz)})
    NTF = dsclansNTF(res['x'], order, rmax, Hz)
    return NTF

def dsclansObja(x, order, OSR, Q, rmax, Hz):
    """Objective function for clans; Optimization Toolbox version >= 6
    f is the magnitude of H at the band-edge
    """
    H = dsclansNTF(x, order, rmax, Hz)
    f = np.abs(evalTF(H, np.exp(1j*np.pi/OSR)))
    return f

def dsclansObjb(x, order, OSR, Q, rmax, Hz):
    """Constraint function for clans; 
    g =||h||_1 - Q
    """
    H = dsclansNTF(x, order, rmax, Hz)
    H = (H[0], H[1], H[2], 1.)
    # dimpulse(H, n=100)[y is 0][output 0]
    g = np.sum(np.abs(dimpulse(H, t=np.arange(100))[1][0])) - 1 - Q
    return -g



# -*- coding: utf-8 -*-
# test_dsclansNTF.py
# This module provides the tests for dsclansNTF() function.
# Copyright 2014 Giuseppe Venturini & Shayne Hodge
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

import unittest
import numpy as np
import deltasigma as ds


class TestDSClansNTF(unittest.TestCase):
    """Test functions for dsclansNTF()"""

    def setUp(self):
        self.x1 = ds.dsclansNTF(np.arange(1, 100.001, .001), 3, .5, 100)

    def test_rp_rt(self):
        """ Test rp == rt  """
        rt = np.array(self.x1[1][:])
        rt.sort()
        rp = np.array([0, -0.016805373426715, 0.014809370415763])
        rp.sort()
        self.assertTrue(np.allclose(rp, rt, rtol=1e-5, atol=1e-6))

    def test_rz_x0(self):
        """ Test rz == x[0] """
        rz = (100., )
        self.assertTrue(np.allclose(rz, self.x1[0], rtol=1e-5, atol=1e-8))

    def test_rp_tp(self):
        """ Test rp == tp """
        rp = np.array(
            [0.35378443 + 0.0j, 0.34187718 + 0.22831019j,
             0.34187718 - 0.22831019j, 0.32978826 + 0.59355161j,
             0.32978826 - 0.59355161j])
        x = np.array([0.67623674, 0.9277613, 0.70365961, 0.60374009,
                      0.78008118])
        Hz = np.array([0.99604531 - 0.08884669j, 0.99604531 + 0.08884669j,
                       0.99860302 - 0.05283948j, 0.99860302 + 0.05283948j,
                       1.00000000 + 0.0j])
        tp = ds.dsclansNTF(x, order=5, rmax=.95, Hz=Hz)[1]
        self.assertTrue(np.allclose(rp, tp, rtol=1e-5, atol=1e-6))



# -*- coding: utf-8 -*-
# test_synthesizeChebyshevNTF.py
# This module provides the tests for the synthesizeChebyshevNTF function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the synthesizeChebyshevNTF() function.
"""

import unittest
import numpy as np
import deltasigma as ds

from warnings import catch_warnings

from nose.tools import raises

from deltasigma._utils import cplxpair

class TestSynthesizeChebyshevNTF(unittest.TestCase):
    """Test class for synthesizeChebyshevNTF()"""

    def setUp(self):
        pass

    def test_synthesizeChebyshevNTF_1(self):
        """Test function for synthesizeChebyshevNTF() 1/3"""
        z, p, k = ds.synthesizeChebyshevNTF()
        zref = [1., .9991 + 0.0425j, .9991 - 0.0425j]
        pref = [.6609, .7686 + .2858j, .7686 - .2858j]
        kref = 1.
        self.assertTrue(np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4, rtol=1e-4))
        self.assertTrue(np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4, rtol=1e-4))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    def test_synthesizeChebyshevNTF_2(self):
        """Test function for synthesizeChebyshevNTF() 2/3"""
        with catch_warnings(record=True) as w:
            z, p, k = ds.synthesizeChebyshevNTF(order=4, OSR=32, opt=1, H_inf=1.5, f0=.33)
            self.assertTrue(len(w) > 0)
        zref = [-.4513 + .8924j, -.4513 - .8924j, -.5122 + 0.8589j, -.5122 - 0.8589j]
        pref = [-.2249 + .7665j, -.2249 - .7665j, -.5506 + .6314j, -.5506 - .6314j]
        kref = 1.
        self.assertTrue(np.allclose(cplxpair(z), cplxpair(zref), atol=1e-4, rtol=1e-4))
        self.assertTrue(np.allclose(cplxpair(p), cplxpair(pref), atol=1e-4, rtol=1e-4))
        self.assertTrue(np.allclose(k, kref, atol=1e-4, rtol=1e-4))

    @raises(ValueError)
    def test_synthesizeChebyshevNTF_3(self):
        """Test function for synthesizeChebyshevNTF() 3/3"""
        z, p, k = ds.synthesizeChebyshevNTF(order=5, OSR=32, opt=0, H_inf=1.5, f0=.33)




# -*- coding: utf-8 -*-
# _evalTFP.py
# Module providing the evalTFP function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

from __future__ import division

import numpy as np

from ._constants import eps
from ._evalRPoly import evalRPoly
from ._evalTF import evalTF
from ._utils import carray, restore_input_form, save_input_form


def evalTFP(Hs, Hz, f):
    """Evaluate a CT-DT transfer function product.

    Compute the value of a transfer function product Hs*Hz at a frequency f,
    where Hs is a continuous-time TF and Hz is a discrete-time TF.

    Use this function to evaluate the signal transfer function of a
    continuous-time (CT) system. In this context ``Hs`` is the open-loop
    response of the loop filter from the ``u`` input and ``Hz`` is the
    closed-loop noise transfer function.

    .. note:: This function attempts to **cancel poles** in ``Hs`` with **zeros** in ``Hz``.

    **Parameters:**

    Hs : tuple
        Hs is a CT (SISO) TF in zpk-tuple form.

    Hz : tuple
        Hz is a DT (SISO) TF in zpk-tuple form.

    f : scalar or 1D array or sequence
        Frequency values.

    **Returns:**

    H : scalar, or ndarray or sequence
        The calculated values: 

        .. math::

            H(f) = H_s(j2\\pi f)\ H_z(e^{j2\\pi f})

        ``H`` has the same form as ``f``.

    .. seealso:: 
        :func:`evalMixedTF`, a more advanced version of this function
        which is used to evaluate the individual feed-in transfer functions of
        a CT modulator.

    **Example:**

    Plot the STF of the 2nd-order CT system depicted at :func:`mapCtoD`.::

        import numpy as np
        import pylab as plt
        from scipy.signal import lti
        from deltasigma import *
        from deltasigma._utils import _get_zpk
        Ac = np.array([[0, 0], [1, 0]])
        Bc = np.array([[1, -1], [0, -1.5]])
        Cc = np.array([[0, 1]])
        Dc = np.array([[0, 0]])
        LFc = (Ac, Bc, Cc, Dc)
        L0c = _get_zpk((Ac, Bc[:, 0].reshape((-1, 1)), Cc, Dc[0, 0].reshape(1, 1)))
        tdac = [0, 1]
        LF, Gp = mapCtoD(LFc, tdac)
        LF = lti(*LF)
        ABCD = np.vstack((np.hstack((LF.A, LF.B)),
                          np.hstack((LF.C, LF.D))
                        ))
        H, _ = calculateTF(ABCD)
        # Yields H=(1-z^-1)^2
        f = np.linspace(0, 2, 300);
        STF = evalTFP(L0c, _get_zpk(H), f)
        plt.figure(figsize=(12, 5))
        plt.plot(f, dbv(STF))
        plt.ylabel("|STF| [dB]")
        plt.xlabel("frequency ($1 \\\\rightarrow f_s$)")
        figureMagic((0, 2), .5, None, (-60, 0), 20, None)

    .. plot::

        import numpy as np
        import pylab as plt
        from scipy.signal import lti
        from deltasigma import *
        from deltasigma._utils import _get_zpk
        Ac = np.array([[0, 0], [1, 0]])
        Bc = np.array([[1, -1], [0, -1.5]])
        Cc = np.array([[0, 1]])
        Dc = np.array([[0, 0]])
        LFc = (Ac, Bc, Cc, Dc)
        L0c = _get_zpk((Ac, Bc[:, 0].reshape((-1, 1)), Cc, Dc[0, 0].reshape(1, 1)))
        tdac = [0, 1]
        LF, Gp = mapCtoD(LFc, tdac)
        LF = lti(*LF)
        ABCD = np.vstack((np.hstack((LF.A, LF.B)),
                          np.hstack((LF.C, LF.D))
                        ))
        H, _ = calculateTF(ABCD)
        # Yields H=(1-z^-1)^2
        f = np.linspace(0, 2, 300);
        STF = evalTFP(L0c, _get_zpk(H), f)
        plt.figure(figsize=(12, 5))
        plt.plot(f, dbv(STF))
        plt.ylabel("|STF| [dB]")
        plt.xlabel("frequency ($1 \\\\rightarrow f_s$)")
        figureMagic((0, 2), .5, None, (-60, 0), 20, None)

    """

    szeros, spoles, sk = Hs
    zzeros, zpoles, zk = Hz
    # sanitize f
    form_f = save_input_form(f)
    f = carray(f)
    if not np.prod(f.shape) == max(f.shape):
        raise ValueError("The f array must have shape " + 
                         "(N,) or (N, 1) or (N, 1, 1) ...")
    f = f.reshape((-1,))
    # sanitize poles and zeros
    szeros, spoles = np.asarray(szeros), np.asarray(spoles)
    zzeros, zzeros = np.asarray(zzeros), np.asarray(zzeros)
    # back to business
    slim = min(0.001, max(1e-05, eps**(1./(1 + spoles.size))))
    zlim = min(0.001, max(1e-05, eps**(1./(1 + zzeros.size))))
    H = np.zeros(f.shape, dtype=np.complex128)
    w = 2*np.pi*f
    s = 1j*w
    z = np.exp(s)
    for i in range(f.shape[0]):
        wi = w[i]
        si = s[i]
        zi = z[i]
        if spoles.size == 0:
            cancel = False
        else:
            cancel = (np.abs(si - spoles) < slim)
        if not np.any(cancel):
            # wi is far from a pole, so just use the product Hs*Hz
            H[i] = evalTF(Hs, si)*evalTF(Hz, zi)
        else:
            # cancel pole(s) of Hs with corresponding zero(s) of Hz
            cancelz = np.abs(zi - zzeros) < zlim
            if np.sum(cancelz) > np.sum(cancel):
                H[i] = 0.
            else:
                if np.sum(cancelz) < np.sum(cancel):
                    H[i] = np.Inf
                else:
                    H[i] = evalRPoly(szeros, si, sk)*zi**np.sum(cancel) \
                           * evalRPoly(zzeros[~cancelz], zi, zk) \
                           / (evalRPoly(spoles[~cancel], si, 1.)  \
                              * evalRPoly(zpoles, zi, 1.))
    # return H matching the shape of f
    H = restore_input_form(H, form_f)
    return H



# -*- coding: utf-8 -*-
# _scaleABCD.py
# Module providing the scaleABCD function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the scaleABCD() function
"""

from __future__ import division, print_function

import numpy as np
import numpy.random as npr

from ._partitionABCD import partitionABCD
from ._simulateDSM import simulateDSM


def scaleABCD(ABCD, nlev=2, f=0, xlim=1, ymax=None, umax=None, N_sim=1e5, N0=10):
    """Scale the loop filter of a general delta-sigma modulator for dynamic range.

    The ABCD matrix is scaled so that the state maxima are less than the
    specified limits (``xlim``). As a side effect, the maximum stable input is
    determined in the process.

    **Parameters:**

    ABCD : ndarray
        The state-space description of the loop filter.

    nlev : int, optional
        The number of levels in the quantizer.

    f : scalar
        The normalized frequency of the test sinusoid.

    xlim : scalar or ndarray
        A vector or scalar specifying the limit for each state variable.

    ymax : scalar, optional
        The stability threshold. Inputs that yield quantizer inputs above ymax
        are considered to be beyond the stable range of the modulator.
        If not provided, it will be set to :math:`n_{lev} + 5`

    umax : scalar, optional
        The maximum allowable input amplitude. ``umax`` is calculated if it
        is not supplied.

    **Returns:**

    ABCDs : ndarray
        The state-space description of the scaled loop filter.

    umax : scalar
        The maximum stable input amplitude. Input sinusoids with amplitudes
        below this value should not cause the modulator states to exceed their
        specified limits.

    S : ndarray
        The diagonal scaling matrix S.

    `S` is defined such that::

        ABCDs = [[S*A*Sinv, S*B], [C*Sinv, D]]
        xs = S*x

    Where the multiplications are *matrix multiplications*.

    """
    if ymax is None:
        ymax = nlev + 5
    order = ABCD.shape[0] - 1
    xlim = xlim*np.ones((order,)) if np.isscalar(xlim) else xlim.reshape((-1, ))
    if np.isreal(ABCD).all():
        quadrature = False
    else:
        quadrature = True
    npr.seed(0) # So that this function is repeatable
    # Envelope for smooth start-up 
    raised_cosine = 0.5*(1 - np.cos(np.pi/N0*np.arange(N0)))

    if umax is None:
        # Simulate the modulator with DC or sine wave inputs to detect its stable
        # input range. 
        # First get a rough estimate of umax.
        ulist = np.arange(0.1, 1.1, 0.1)*(nlev - 1)
        umax = nlev - 1
        N = 1000.0
        u0 = np.hstack((
                        np.exp(2j*np.pi*f*np.arange(- N0, 0))*raised_cosine, \
                        np.exp(2j*np.pi*f*np.arange(0, N))
                      )) \
              + 0.01*np.dot(np.array([[1, 1j]]), npr.randn(2, N + N0))
        if not quadrature:
            u0 = np.real(u0)
        for u in ulist:
            if not quadrature:
                v, x, xmax, y = simulateDSM(u*u0, ABCD, nlev)
            else:
                raise NotImplementedError("simulateQDSM has not been implemented yet.")
                #v, x, xmax, y = simulateQDSM(u*u0, ABCD, nlev)
            if np.max(np.abs(y)) > ymax:
                umax = u
                # umax is the smallest input found which causes 'instability'
                break
        if umax == ulist[0]:
            msg = 'Modulator is unstable even with an input amplitude of %.1f.'\
             % umax
            raise RuntimeError(msg)
            
    # More detailed simulation
    N = N_sim
    u0 = np.hstack((
                    np.exp(2j*np.pi*f*np.arange(-N0, 0))*raised_cosine, \
                    np.exp(2j*np.pi*f*np.arange(0, N))
                  )) \
            + 0.01*np.dot(np.array([[1, 1j]]), npr.randn(2, N + N0))
    if not quadrature:
        u0 = np.real(u0)
    maxima = np.zeros((1, order)) - 1
    ulist = np.linspace(0.7*umax, umax, 10)
    for u in ulist:
        if not quadrature:
            v, x, xmax, y = simulateDSM(u*u0, ABCD, nlev)
        else:
            raise NotImplementedError("simulateQDSM has not been implemented yet.")
            #v, x, xmax, y = simulateQDSM(u*u0, ABCD, nlev)
        if np.max(np.abs(y)) > ymax:
            break
        umax = u
        maxima = np.max(np.vstack((maxima, xmax.T)), axis=0, keepdims=True)
    scale = (maxima/xlim).reshape((-1))
    S = np.diag(1.0/scale)
    Sinv = np.diag(scale)
    A, B, C, D = partitionABCD(ABCD)
    ABCDs = np.vstack((
                       np.hstack((np.dot(np.dot(S, A), Sinv), np.dot(S, B))),
                       np.hstack((np.dot(C, Sinv), D))
                     ))
    return ABCDs, umax, S



# -*- coding: utf-8 -*-
# _config.py
# Module providing configuration switches
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing configuration switches.
"""

import os
import sys
from warnings import warn

import numpy as np
from numpy.distutils.system_info import get_info

# should synthesizeNTF run the optimization routine?
optimize_NTF = True

# how many iterations should be allowed in NTF synthesis?
# see synthesizeNTF() for more
itn_limit = 500

# debug
_debug = False

# get blas information to compile the cython extensions
blas_info = get_info("blas")
if len(blas_info) == 0 and _debug:
    warn("Numpy did not detect the BLAS library in the system")
# Let's make an educated guess
if 'linux' in sys.platform or 'darwin' in sys.platform:
    guessed_include = '/usr/include'
else:
    guessed_include = None
# wrap it up: numpy or user-set environment var or a lucky guess on our side is
# needed to get the cblas.h header path. If not found, simulateDSM() will use
# a CPython implementation (slower).
setup_args = {"script_args":(["--compiler=mingw32"]
                             if sys.platform == 'win32' else [])}
lib_include = [np.get_include()]
if "include_dirs" not in blas_info and "BLAS_H" not in os.environ and \
   'nt' not in os.name and (not guessed_include or \
     not os.path.isfile(os.path.join(guessed_include, 'cblas.h'))):
    warn("Cannot find the path for 'cblas.h'. You may set it using the environment variable "
         "BLAS_H.\nNOTE: You need to pass the path to the directories were the "
         "header files are, not the path to the files.")
else:
    if "include_dirs" in blas_info:
        lib_include = lib_include + blas_info.get("include_dirs")
    elif "BLAS_H" in os.environ:
        lib_include = lib_include + [os.environ["BLAS_H"]]
    elif guessed_include:
        lib_include = lib_include + [guessed_include]
    else:
        pass # we're on windows
setup_args.update({"include_dirs":list(set(lib_include))})



# -*- coding: utf-8 -*-
# test_impL1.py
# Test module for the impL1() function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Test module for the impL1() function.
"""

import unittest
import numpy as np
import deltasigma as ds

from scipy.signal import lti

class TestTestTemplate(unittest.TestCase):
    """Test class for the impL1 function"""
    def setUp(self):
        self.r2 = np.array([0., 0.4, -0.16, 0.064, -0.0256, 0.01024,
                            -0.004096, 0.0016384, -0.00065536,
                            0.000262144, -0.0001048576])

    def test_impL1_zpk(self):
        """Test function for impL1() 1/4"""
        sys1 = (np.array([-.4]), np.array([0, 0]), 1) # zpk
        r1 = ds.impL1(sys1, n=10)
        self.assertTrue(np.allclose(r1, self.r2, atol=1e-8, rtol=1e-4))

    def test_impL1_num_den(self):
        """Test function for impL1() 2/4"""
        sys1 = (np.array([-.4]), np.array([0, 0]), 1) 
        num = np.poly(sys1[0])
        den = np.poly(sys1[1])
        num[0] *= sys1[2]
        tf = (num, den)
        r3 = ds.impL1(tf, n=10)
        self.assertTrue(np.allclose(self.r2, r3, atol=1e-8, rtol=1e-4))

    def test_impL1_lti(self):
        """Test function for impL1() 3/4"""
        sys1 = (np.array([-.4]), np.array([0, 0]), 1)
        tf = lti(*sys1)
        r4 = ds.impL1(tf, n=10)
        self.assertTrue(np.allclose(self.r2, r4, atol=1e-8, rtol=1e-4))

    def test_impL1_cplx(self):
        """Test function for impL1() 4/4"""
        res = np.array([0.00000000000000 + 0.00000000000000j,
                        0.333000000000000 - 0.333000000000000j,
                        -0.110889000000000 + 0.110889000000000j,
                        0.0369260370000000 - 0.0369260370000000j,
                        -0.0122963703210000 + 0.0122963703210000j,
                        0.00409469131689300 - 0.00409469131689300j,
                        -0.00136353220852537 + 0.00136353220852537j,
                        0.000454056225438948 - 0.000454056225438948j,
                        -0.000151200723071170 + 0.000151200723071170j,
                        5.03498407826996e-05 - 5.03498407826996e-05j,
                        -1.67664969806390e-05 + 1.67664969806390e-05j])

        a = [[-.3333], [-.3333j], 1]
        y = ds.impL1(a)
        self.assertTrue(np.allclose(y, res, atol=1e-3, rtol=1e-4)) 



# -*- coding: utf-8 -*-
# _realizeNTF_ct.py
# Module providing the realizeNTF_ct function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the realizeNTF_ct() function
"""

from __future__ import division, print_function

from warnings import warn

import numpy as np
import numpy.linalg as linalg
from scipy.signal import dimpulse, ss2zpk

from ._evalTFP import evalTFP
from ._impL1 import impL1
from ._padb import padb
from ._pulse import pulse
from ._utils import _get_zpk, carray, eps


def realizeNTF_ct(ntf, form='FB', tdac=(0, 1), ordering=None, bp=None,
                  ABCDc=None, method='LOOP'):
    """Realize an NTF with a continuous-time loop filter.

    **Parameters:**

    ntf : object
        A noise transfer function (NTF).

    form : str, optional
        A string specifying the topology of the loop filter.

         * 'FB': Feedback form,
         * 'FF': Feedforward form

        For the FB structure, the elements of ``Bc`` are calculated
        so that the sampled pulse response matches the L1 impulse
        response.  For the FF structure, ``Cc`` is calculated.

    tdac : sequence, optional
        The timing for the feedback DAC(s). If ``tdac[0] >= 1``,
        direct feedback terms are added to the quantizer.

        Multiple timings (one or more per integrator) for the FB
        topology can be specified by making tdac a list of lists,
        e.g. ``tdac = [[1, 2], [1, 2], [[0.5, 1], [1, 1.5]], []]``

        In this example, the first two integrators have
        DACs with ``[1, 2]`` timing, the third has a pair of
        DACs, one with ``[0.5, 1]`` timing and the other with
        ``[1, 1.5]`` timing, and there is no direct feedback
        DAC to the quantizer.

    ordering : sequence, optional
        A vector specifying which NTF zero-pair to use in each resonator
        Default is for the zero-pairs to be used in the order specified
        in the NTF.

    bp : sequence, optional
        A vector specifying which resonator sections are bandpass.
        The default (``zeros(...)``) is for all sections to be lowpass.

    ABCDc : ndarray, optional
        The loop filter structure, in state-space form.
        If this argument is omitted, ABCDc is constructed according
        to "form."

    method : str, optional
        The default fitting method is ``'LOOP'``, which means that
        the DT and CT loop responses will be matched.
        Alternatively, it is possible to set the method to ``'NTF'``,
        which will result in the NTF responses to be matched.
        See :ref:`discrete-time-to-continuous-time-mapping` for a
        more in-depth discussion.

    **Returns:**

    ABCDc : ndarray
        A state-space description of the CT loop filter

    tdac2 : ndarray
        A matrix with the DAC timings, including ones
        that were automatically added.

    **Example:**

    Realize the NTF :math:`(1 - z^{-1})^2` with a CT system (cf with the
    example at :func:`mapCtoD`).::

        from deltasigma import *
        ntf = ([1, 1], [0, 0], 1)
        ABCDc, tdac2 = realizeNTF_ct(ntf, 'FB')

    Returns:

    ABCDc::

        [[ 0.          0.          1.         -1.        ]
         [ 1.          0.          0.         -1.49999999]
         [ 0.          1.          0.          0.        ]]

    tdac2::

        [[-1. -1.]
         [ 0.  1.]]

    """
    ntf_z, ntf_p, _ = _get_zpk(ntf)
    ntf_z = carray(ntf_z)
    ntf_p = carray(ntf_p)
    order = max(ntf_p.shape)
    order2 = int(np.floor(order/2.))
    odd = order - 2*order2
    # compensate for limited accuracy of zero calculation
    ntf_z[np.abs(ntf_z - 1) < eps**(1./(1. + order))] = 1.
    method = method.upper()
    if method not in ('LOOP', 'NTF'):
        raise ValueError('Unimplemented matching method %s.' % method)
    # check if multiple timings mode
    if (type(tdac) == list or type(tdac) == tuple) and len(tdac) and \
       (type(tdac[0]) == list or type(tdac[0]) == tuple):
        if len(tdac) != order + 1:
            msg = 'For multi-timing tdac, len(tdac) ' + \
                  ' must be order+1.'
            raise ValueError(msg)
        if form != 'FB':
            msg = "Currently only supporting form='FB' " + \
                  'for multi-timing tdac'
            raise ValueError(msg)
        multi_timing = True
    else: # single timing
        tdac = carray(tdac)
        if np.prod(tdac.shape) != 2:
            msg = 'For single-timing tdac, len(tdac) must be 2.'
            raise ValueError(msg)
        tdac.reshape((2,))
        multi_timing = False
    if ordering is None:
        ordering = np.arange(order2)
    if bp is None:
        bp = np.zeros((order2,))
    if not multi_timing:
        # Need direct terms for every interval of memory in the DAC
        n_direct = np.ceil(tdac[1]) - 1
        if tdac[0] > 0 and tdac[0] < 1 and tdac[1] > 1 and tdac[1] < 2:
            n_extra = n_direct - 1 #  tdac pulse spans a sample point
        else:
            n_extra = n_direct
        tdac2 = np.vstack(
                 (np.array((-1, -1)),
                  np.array(tdac).reshape((1, 2)),
                  0.5*np.dot(np.ones((n_extra, 1)), np.array([[-1, 1]]))
                  + np.cumsum(np.ones((n_extra, 2)), 0) + (n_direct - n_extra)
                 ))
    else:
        n_direct = 0
        n_extra = 0
    if ABCDc is None:
        ABCDc = np.zeros((order + 1, order + 2))
        # Stuff the A portion
        if odd:
            ABCDc[0, 0] = np.real(np.log(ntf_z[0]))
            ABCDc[1, 0] = 1
        dline = np.array([0, 1, 2])
        for i in range(order2):
            n = bp[i]
            i1 = 2*i + odd
            zi = 2*ordering[i] + odd
            w = np.abs(np.angle(ntf_z[zi]))
            ABCDc[i1 + dline, i1] = np.array([0, 1, n])
            ABCDc[i1 + dline, i1 + 1] = np.array([-w**2, 0, 1 - n])
        ABCDc[0, order] = 1
        # 2006.10.02 Changed to -1 to make FF STF have +ve gain at DC
        ABCDc[0, order + 1] = -1
    Ac = ABCDc[:order, :order]
    if form == 'FB':
        Cc = ABCDc[order, :order].reshape((1, -1))
        if not multi_timing:
            Bc = np.hstack((np.eye(order), np.zeros((order, 1))))
            Dc = np.hstack((np.zeros((1, order)), np.array([[1]])))
            tp = np.tile(np.array(tdac).reshape((1, 2)), (order + 1, 1))
        else: #Assemble tdac2, Bc and Dc
            tdac2 = np.array([[-1, -1]])
            Bc = None
            Dc = None
            Bci = np.hstack((np.eye(order), np.zeros((order, 1))))
            Dci = np.hstack((np.zeros((1, order)), np.array([[1]])))
            for i in range(len(tdac)):
                tdi = tdac[i]
                if (type(tdi) in (tuple, list)) and len(tdi) and \
                   (type(tdi[0]) in (list, tuple)):
                    for j in range(len(tdi)):
                        tdj = tdi[j]
                        tdac2 = np.vstack((tdac2,
                                           np.array(tdj).reshape(1,-1)))
                        if Bc is not None:
                            Bc = np.hstack((Bc, Bci[:, i].reshape((-1, 1))))
                        else:
                            Bc = Bci[:, i].reshape((-1, 1))
                        if Dc is not None:
                            Dc = np.hstack((Dc, Dci[:, i].reshape((-1, 1))))
                        else:
                            Dc = Dci[:, i].reshape((-1, 1))
                elif len(tdi): # we got tdac[i] = [a, b] where a, b are scalars
                    tdac2 = np.vstack((tdac2,
                                       np.array(tdi).reshape(1,-1)))
                    if Bc is not None:
                        Bc = np.hstack((Bc, Bci[:, i].reshape((-1, 1))))
                    else:
                        Bc = Bci[:, i].reshape((-1, 1))
                    if Dc is not None:
                        Dc = np.hstack((Dc, Dci[:, i].reshape((-1, 1))))
                    else:
                        Dc = Dci[:, i].reshape((-1, 1))
            tp = tdac2[1:, :]
    elif form == 'FF':
        Cc = np.vstack((np.eye(order), np.zeros((1, order))))
        Bc = np.vstack((np.array([[-1]]), np.zeros((order-1, 1))))
        Dc = np.vstack((np.zeros((order, 1)), np.array([[1]])))
        tp = tdac #  2008-03-24 fix from Ayman Shabra
    else:
        raise ValueError('Sorry, no code for form "%s".', form)

    n_imp = np.ceil(2*order + np.max(tdac2[:, 1]) + 1)
    if method == 'LOOP':
        # Sample the L1 impulse response
        y = impL1(ntf, n_imp)
    else:
        # Sample the NTF impulse response
        y = dimpulse((ntf_z, ntf_p, 1., 1.), t=np.arange(n_imp+1))[1][0]
        y = np.atleast_1d(y.squeeze())
    sys_c = []
    for i in range(Bc.shape[1]): # number of inputs
        sys_tmp = []
        for j in range(Cc.shape[0]): # number of outputs
            sys_tmp.append(ss2zpk(Ac, Bc, Cc[j, :], Dc[j, :], input=i))
        sys_c.append(sys_tmp)
    yy = pulse(sys_c, tp, 1, n_imp, 1)
    yy = np.squeeze(yy)
    # Endow yy with n_extra extra impulses.
    # These will need to be implemented with n_extra extra DACs.
    # !! Note: if t1=int, matlab says pulse(sys) @t1 ~=0
    # !! This code corrects this problem.
    if n_extra > 0:
        y_right = padb(np.vstack((np.zeros((1, n_direct)),
                                  np.eye(n_direct))),
                       n_imp + 1)
        # Replace the last column in yy with an ordered set of impulses
        if (n_direct > n_extra):
            yy = np.hstack((yy, y_right[:, 1:]))
        else:
            yy = np.hstack((yy[:, :-1], y_right))

    if method == 'NTF':
        # convolve CT loop response and NTF response
        yynew = None
        for i in range(yy.shape[1]):
            yytmp = np.convolve(yy[:, i], y)[:-n_imp]
            if yynew is None:
                yynew = yytmp.reshape((-1, 1))
            else:
                yynew = np.hstack((yynew, yytmp.reshape((-1, 1))))
        yy = yynew
        e1 = np.zeros(y.shape)
        e1[0] = 1.
        y = y - e1
    # Solve for the coefficients
    x = linalg.lstsq(yy, y)[0]
    if linalg.norm(np.dot(yy, x) - y) > 0.0001:
        warn('Pulse response fit is poor.')
    if form == 'FB':
        if not multi_timing:
            Bc2 = np.hstack((x[:order].reshape((-1, 1)),
                             np.zeros((order, n_extra))))
            if n_extra > 0:
                Dc2 = np.hstack((np.array([[0]]),
                                 x[order:].reshape((-1, 1))))
            else:
                Dc2 = x[order:].reshape((-1, 1))
        else:
            BcDc = np.vstack((Bc, Dc))
            i = np.nonzero(BcDc)
            BcDc[i] = x
            Bc2 = BcDc[:-1, :]
            Dc2 = BcDc[-1, :]
    elif form == 'FF':
        Bc2 = np.hstack((Bc, np.zeros((order, n_extra))))
        Cc = x[:order].reshape((1, -1))
        if n_extra > 0:
            Dc2 = np.hstack((np.array([[0]]), x[order:].T))
        else:
            Dc2 = x[order:].T

    Dc1 = np.zeros((1, 1))
    Dc = np.hstack((Dc1, np.atleast_2d(Dc2)))
    Bc1 = np.vstack((np.ones((1, 1)), np.zeros((order - 1, 1))))
    Bc = np.hstack((Bc1, Bc2))
    # Scale Bc1 for unity STF magnitude at f0
    fz = np.angle(ntf_z)/(2*np.pi)
    f1 = fz[0]
    ibz = np.abs(fz - f1) <= np.abs(fz + f1)
    fz = fz[ibz]
    f0 = np.mean(fz)
    if np.min(np.abs(fz)) < 3*np.min(np.abs(fz - f0)):
        f0 = 0
    L0c = ss2zpk(Ac, Bc1, Cc, Dc1)
    G0 = evalTFP(L0c, ntf, f0)
    if f0 == 0:
        Bc[:, 0] = np.dot(Bc[:, 0],
                          np.abs(np.dot(Bc[0, 1:],
                                        (tdac2[1:, 1] - tdac2[1:, 0]))
                                 /Bc[0, 0]))
    else:
        Bc[:, 0] = Bc[:, 0]/np.abs(G0)
    ABCDc = np.vstack((
                      np.hstack((Ac, Bc)),
                      np.hstack((Cc, Dc))
                     ))
    #ABCDc = np.dot(ABCDc, np.abs(ABCDc) > eps**(1./2.))
    ABCDc[np.nonzero(np.abs(ABCDc) < eps**(1./2))] = 0.
    return ABCDc, tdac2



# -*- coding: utf-8 -*-
# test_dbm.py
# This module provides the tests for the dbm function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the dbm() function.
"""

import unittest
import numpy as np

from deltasigma import dbm

class TestDbm(unittest.TestCase):
    """Test class for dbm()"""
    def setUp(self):
        self.v1 = np.arange(10) * 1e-3 # test arrays
        self.r1 = [-np.inf, -46.98970004, -40.96910013, -37.44727495, -34.94850022,
                   -33.01029996, -31.42667504, -30.08773924, -28.9279003, -27.90484985]
        self.v2 = 9e-3  # test scalars.
        self.r2 = -27.90484985

    def test_dbm_1(self):
        """Test function for dbm() 1/2"""
        self.assertTrue(np.allclose(dbm(self.v1), self.r1, atol=1e-8, rtol=1e-5))

    def test_dbm_2(self):
        """Test function for dbm() 2/2"""
        self.assertTrue(np.allclose(dbm(self.v2), self.r2, atol=1e-8, rtol=1e-5))
        self.assertTrue(np.isscalar(dbm(self.v2)))




# -*- coding: utf-8 -*-
# test_plotSpectrum.py
# This module provides the tests for the plotSpectrum function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the plotSpectrum() function.
"""

import unittest
import numpy as np
import deltasigma as ds
import pylab as plt

from numpy.fft import fft

class TestPlotSpectrum(unittest.TestCase):
    """Test class for plotSpectrum()"""

    def setUp(self):
        pass

    def test_plotSpectrum(self):
        """Test function for plotSpectrum()"""
        f0 = 0
        osr = 32
        quadrature = False
        Hinf = 1.5
        order = 3
        ntf = ds.synthesizeNTF(order, osr, 0, Hinf, f0)
        f1, f2 = ds.ds_f1f2(osr, f0, quadrature)
        delta = 2
        Amp = ds.undbv(-3)
        f = 0.3
        N = 2**12
        f1_bin = np.round(f1*N)
        f2_bin = np.round(f2*N)
        fin = np.round(((1 - f)/2*f1 + (f + 1)/2*f2) * N)
        t = np.arange(0, N)
        u = Amp*np.cos((2*np.pi/N)*fin*t)
        v, xn, xmax, y = ds.simulateDSM(u, ntf, 2)
        window = ds.ds_hann(N)
        NBW = 1.5/N
        spec0 = fft(v * window)/(N/4)
        freq = np.linspace(0, 0.5, N/2 + 1)
        # plotting
        plt.subplot(211)
        plt.plot(freq, ds.dbv(spec0[:N/2 + 1]), 'c', linewidth=1, label='$S$')
        plt.hold(True)
        spec_smoothed = ds.circ_smooth(np.abs(spec0)**2., 16)
        plt.plot(freq, ds.dbp(spec_smoothed[:N/2 + 1]), 'b--', linewidth=2,
                 label='$\\mathrm{circ\\_smooth}(S)$')
        ds.plotSpectrum(spec0, fin, 'r', linewidth=2,
                        label='$\\mathrm{plotSpectrum}(S)$')
        Snn = np.abs(ds.evalTF(ntf, np.exp(2j*np.pi*freq)))**2 * 2/12*(delta)**2
        plt.plot(freq, ds.dbp(Snn*NBW), 'm', linewidth=1.5,
                 label='$\mathrm{from\\ NTF}$')
        plt.text(0.5, -3, 'NBW = %.1e ' % NBW, horizontalalignment='right',
                 verticalalignment='top')
        ds.figureMagic((0, 0.5), None, None, (-140, 0), 20, None)
        plt.ylabel('Spectrum [dB]')
        ax = plt.gca()
        ax.set_title('Smoothing and plotting for LOG and LIN axes')
        plt.legend(loc=4)
        plt.subplot(212)
        plt.plot(freq, ds.dbv(spec0[:N/2 + 1]), 'c', linewidth=1, label='$S$')
        plt.hold(True)
        ds.plotSpectrum(spec0, fin, '--r', linewidth=2,
                        label='$\\mathrm{plotSpectrum}(S)$')
        plt.plot(freq, ds.dbp(spec_smoothed[:N/2 + 1]), 'b', linewidth=2,
                 label='$\\mathrm{circ\\_smooth}(S)$')
        plt.plot(freq, ds.dbp(Snn*NBW), 'm', linewidth=1.5,
                 label='$\mathrm{from\\ NTF}$')
        plt.text(0.5, -3, 'NBW = %.1e ' % NBW, horizontalalignment='right',
                 verticalalignment='top')
        ds.figureMagic((0, 0.5), None, None, (-140, 0), 20, None)
        ax = plt.gca()
        ax.set_xscale('linear')
        plt.ylabel('Spectrum [dB]')
        plt.xlabel('Normalized frequency ($f_s \\rightarrow 1$)')
        plt.legend(loc=4)




# -*- coding: utf-8 -*-
# test_simulateDSM.py
# This module provides the tests for the simulateDSM function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the simulateDSM() function.
"""

import unittest
import pkg_resources

import numpy as np
import deltasigma as ds
import scipy.io

from deltasigma import synthesizeNTF, realizeNTF, stuffABCD, cplxpair

class TestSimulateDSM(unittest.TestCase):
    """Test class for simulateDSM()"""

    def setUp(self):
        fname = pkg_resources.resource_filename(__name__,
                "test_data/test_simulateDSM.mat")
        self.v_ref = scipy.io.loadmat(fname)['v']
        self.xn_ref = scipy.io.loadmat(fname)['xn']
        self.xmax_ref = scipy.io.loadmat(fname)['xmax']
        self.y_ref = scipy.io.loadmat(fname)['y']
        self.u_ref = scipy.io.loadmat(fname)['u']
        self.ABCD_ref = scipy.io.loadmat(fname)['ABCD']
        self.zeros = cplxpair(scipy.io.loadmat(fname)['zeros'])
        self.poles = cplxpair(scipy.io.loadmat(fname)['poles'])
        self.v_ref = self.v_ref.reshape(-1)
        self.y_ref = self.y_ref.reshape(-1)
        OSR = 32
        self.H = synthesizeNTF(5, OSR, 10)
        N = 8192
        f = 85
        self.u = 0.5*np.sin(2*np.pi*f/N*np.arange(N))
        a, g, b, c = realizeNTF(self.H, 'CRFB')
        self.ABCD = stuffABCD(a, g, b, c, form='CRFB')

    def test_precheck(self):
        """Test function for simulateDSM_* preliminary 1/1"""
        from deltasigma._utils import _get_zpk
        zeros, poles, _ = _get_zpk(self.H)
        assert np.allclose(cplxpair(zeros), self.zeros, atol=1e-6, rtol=1e-4)
        assert np.allclose(cplxpair(poles), self.poles, atol=1e-6, rtol=1e-4)
        assert np.allclose(self.ABCD, self.ABCD_ref, atol=1e-5, rtol=1e-3)
        assert np.allclose(self.u, self.u_ref, atol=1e-5, rtol=1e-3)

    def test_simulateDSM_cblas(self):
        """Test function for simulateDSM_cblas()"""
        # skip if not available
        try:
            from nose.plugins.skip import SkipTest
        except ImportError:
            SkipTest = None
        if ds._simulateDSM._simulateDSM_cblas is None:
            if SkipTest is not None:
                raise SkipTest
            return
        v, _, _, y = ds._simulateDSM._simulateDSM_cblas(self.u, self.H, nlev=10,
                                                        x0=0.0, store_xn=True,
                                                        store_xmax=True,
                                                        store_y=True)
        v = v.reshape(-1)
        y = y.reshape(-1)
        self.assertTrue(np.allclose(v[:1200], self.v_ref[:1200], atol=1e-6,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(y[:1200], self.y_ref[:1200], atol=1e-1,
                                    rtol=1e-2))
        v, xn, xmax, y = ds._simulateDSM._simulateDSM_cblas(self.u, self.ABCD,
                                                 nlev=10, x0=0, store_xn=True,
                                                store_xmax=True, store_y=True)
        v = v.reshape(-1)
        y = y.reshape(-1)
        self.assertTrue(np.allclose(v[:1200], self.v_ref[:1200], atol=1e-6,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(y[:1200], self.y_ref[:1200], atol=1e-1,
                                    rtol=1e-3))
        self.assertTrue(np.allclose(xmax, self.xmax_ref, atol=1e-1, rtol=1e-1))
        self.assertTrue(np.allclose(xn[:, :1200], self.xn_ref[:, :1200],
                                    atol=1e-1, rtol=1e-3))

    def test_simulateDSM_scipy_blas(self):
        """Test function for simulateDSM_scipy_blas()"""
        # skip if not available
        try:
            from nose.plugins.skip import SkipTest
        except ImportError:
            SkipTest = None
        if ds._simulateDSM._simulateDSM_scipy_blas is None:
            if SkipTest is not None:
                raise SkipTest
            return
        v, xn, xmax, y = ds._simulateDSM._simulateDSM_scipy_blas(self.u, self.H,
                                                  nlev=10, x0=0., store_xn=True,
                                                  store_xmax=True, store_y=True)
        v = v.reshape(-1)
        y = y.reshape(-1)
        self.assertTrue(np.allclose(v[:1200], self.v_ref[:1200], atol=1e-6,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(y[:1200], self.y_ref[:1200], atol=1e-1,
                                    rtol=1e-2))
        v, xn, xmax, y = ds._simulateDSM._simulateDSM_scipy_blas(self.u,
                                    self.ABCD, nlev=10, x0=0, store_xn=True,
                                    store_xmax=True, store_y=True)
        v = v.reshape(-1)
        y = y.reshape(-1)
        self.assertTrue(np.allclose(v[:1200], self.v_ref[:1200], atol=1e-6,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(y[:1200], self.y_ref[:1200], atol=1e-1,
                                    rtol=1e-3))
        self.assertTrue(np.allclose(xmax, self.xmax_ref, atol=1e-1, rtol=1e-1))
        self.assertTrue(np.allclose(xn[:, :1200], self.xn_ref[:, :1200],
                                    atol=1e-1, rtol=1e-3))

    def test_simulateDSM_python(self):
        """Test function for simulateDSM_python()"""
        v, xn, xmax, y = ds._simulateDSM._simulateDSM_python(self.u, self.H,
                                                             nlev=10., x0=0.)
        v = v.reshape(-1)
        y = y.reshape(-1)
        self.assertTrue(np.allclose(v[:1200], self.v_ref[:1200], atol=1e-6,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(y[:1200], self.y_ref[:1200], atol=1e-1,
                                    rtol=1e-2))
        v, xn, xmax, y = ds._simulateDSM._simulateDSM_python(self.u, self.ABCD,
                                                             nlev=10, x0=0.)
        v = v.reshape(-1)
        y = y.reshape(-1)
        self.assertTrue(np.allclose(v[:1200], self.v_ref[:1200], atol=1e-6,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(y[:1200], self.y_ref[:1200], atol=1e-1,
                                    rtol=1e-3))
        self.assertTrue(np.allclose(xmax, self.xmax_ref, atol=1e-1, rtol=1e-1))
        self.assertTrue(np.allclose(xn[:, :1200], self.xn_ref[:, :1200],
                                    atol=1e-1, rtol=1e-3))

    def test_simulateDSM_scipy_cblas(self):
        """Test function for scipy vs cblas"""
        # skip if not available
        try:
            from nose.plugins.skip import SkipTest
        except ImportError:
            SkipTest = None
        if ds._simulateDSM._simulateDSM_scipy_blas is None or \
            ds._simulateDSM._simulateDSM_cblas is None:
            if SkipTest is not None:
                raise SkipTest
            return
        vb, _, _, yb = ds._simulateDSM._simulateDSM_scipy_blas(self.u,
                                    self.H, nlev=10, x0=0.,
                                    store_xn=True, store_xmax=True,
                                    store_y=True)
        vc, _, _, yc = ds._simulateDSM._simulateDSM_cblas(self.u, self.H,
                                    nlev=10, x0=0., store_xn=True,
                                    store_xmax=True, store_y=True)
        self.assertTrue(np.allclose(vb, vc, atol=1e-6, rtol=1e-4))
        self.assertTrue(np.allclose(yb, yc, atol=1e-6, rtol=1e-4))
        vb, xnb, xmaxb, yb = ds._simulateDSM._simulateDSM_scipy_blas(self.u,
                                    self.ABCD, nlev=10, x0=0., store_xn=True,
                                    store_xmax=True, store_y=True)
        vc, xnc, xmaxc, yc = ds._simulateDSM._simulateDSM_cblas(self.u,
                                    self.ABCD, nlev=10, x0=0., store_xn=True,
                                    store_xmax=True, store_y=True)
        self.assertTrue(np.allclose(vb, vc, atol=1e-6, rtol=1e-4))
        self.assertTrue(np.allclose(xnb, xnc, atol=1e-6, rtol=1e-4))
        self.assertTrue(np.allclose(xmaxb, xmaxc, atol=1e-6, rtol=1e-4))
        self.assertTrue(np.allclose(yb, yc, atol=1e-6, rtol=1e-4))

    def test_simulateDSM_python_scipy(self):
        """Test function for Python vs scipy"""
        # skip if not available
        try:
            from nose.plugins.skip import SkipTest
        except ImportError:
            SkipTest = None
        if ds._simulateDSM._simulateDSM_scipy_blas is None:
            if SkipTest is not None:
                raise SkipTest
            return
        vb, _, _, yb = ds._simulateDSM._simulateDSM_scipy_blas(self.u,
                                    self.H, nlev=10, x0=0.,
                                    store_xn=True, store_xmax=True,
                                    store_y=True)
        vc, _, _, yc = ds._simulateDSM._simulateDSM_python(self.u, self.H,
                                    nlev=10, x0=0.)
        self.assertTrue(np.allclose(vb, vc, atol=1e-6, rtol=1e-4))
        self.assertTrue(np.allclose(yb, yc, atol=1e-6, rtol=1e-4))
        vb, xnb, xmaxb, yb = ds._simulateDSM._simulateDSM_scipy_blas(self.u,
                                    self.ABCD, nlev=10, x0=0., store_xn=True,
                                    store_xmax=True, store_y=True)
        vc, xnc, xmaxc, yc = ds._simulateDSM._simulateDSM_python(self.u,
                                    self.ABCD, nlev=10, x0=0.)
        self.assertTrue(np.allclose(vb, vc, atol=1e-6, rtol=1e-4))
        self.assertTrue(np.allclose(xnb, xnc, atol=1e-6, rtol=1e-4))
        self.assertTrue(np.allclose(xmaxb, xmaxc, atol=1e-6, rtol=1e-4))
        self.assertTrue(np.allclose(yb, yc, atol=1e-6, rtol=1e-4))

    def test_simulateDSM_python_cblas(self):
        """Test function for Python vs cblas"""
        # skip if not available
        try:
            from nose.plugins.skip import SkipTest
        except ImportError:
            SkipTest = None
        if ds._simulateDSM._simulateDSM_cblas is None:
            if SkipTest is not None:
                raise SkipTest
            return
        vb, _, _, yb = ds._simulateDSM._simulateDSM_cblas(self.u,
                                    self.H, nlev=10, x0=0.,
                                    store_xn=True, store_xmax=True,
                                    store_y=True)
        vc, _, _, yc = ds._simulateDSM._simulateDSM_python(self.u, self.H,
                                    nlev=10, x0=0.)
        self.assertTrue(np.allclose(vb, vc, atol=1e-6, rtol=1e-4))
        self.assertTrue(np.allclose(yb, yc, atol=1e-6, rtol=1e-4))
        vb, xnb, xmaxb, yb = ds._simulateDSM._simulateDSM_cblas(self.u,
                                    self.ABCD, nlev=10, x0=0., store_xn=True,
                                    store_xmax=True, store_y=True)
        vc, xnc, xmaxc, yc = ds._simulateDSM._simulateDSM_python(self.u,
                                    self.ABCD, nlev=10, x0=0.)
        self.assertTrue(np.allclose(vb, vc, atol=1e-6, rtol=1e-4))
        self.assertTrue(np.allclose(xnb, xnc, atol=1e-6, rtol=1e-4))
        self.assertTrue(np.allclose(xmaxb, xmaxc, atol=1e-6, rtol=1e-4))
        self.assertTrue(np.allclose(yb, yc, atol=1e-6, rtol=1e-4))




# -*- coding: utf-8 -*-
# test_clans.py
# This module provides the tests for the clans() function.
# Copyright 2014 Giuseppe Venturini & Shayne Hodge
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

import unittest
import numpy as np
import deltasigma as ds


class TestClans(unittest.TestCase):
    """Class doc string"""
    def setUp(self):
        self.ntf = ds.clans(5, 32, 5, .95, 1)
        self.poles = np.array((0.41835234+0.0j, 0.48922229+0.1709716j,
                               0.48922229-0.1709716j, 0.65244885+0.3817224j,
                               0.65244885-0.3817224j))

    def test_clans(self):
        """Test function for clans()"""
        self.assertTrue(
            np.allclose(self.poles, self.ntf[1], atol=1e-8, rtol=1e-5))



# -*- coding: utf-8 -*-
# test_l1norm.py
# This module provides the tests for the l1norm function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the l1norm() function.
"""

import unittest
import numpy as np
import deltasigma as ds

from scipy.signal import lti

class Testl1norm(unittest.TestCase):
    """Test class for l1norm()"""

    def setUp(self):
        zeros = np.array(())
        poles = np.array((.5,))
        k = 1.
        self.zpk_tuple = zeros, poles, k
        splti = lti(zeros, poles, k)
        self.num_den_tuple = (splti.num, splti.den)
        self.ABCD_tuple = (splti.A, splti.B, splti.C, splti.D)
        self.splti = splti

    def test_l1norm_1(self):
        """Test function for l1norm() 1/4"""
        self.assertTrue(np.allclose(ds.l1norm(self.num_den_tuple), 2.,
                        rtol=1e-5, atol=1e-8))

    def test_l1norm_2(self):
        """Test function for l1norm() 2/4"""
        self.assertTrue(np.allclose(ds.l1norm(self.zpk_tuple), 2.,
                        rtol=1e-5, atol=1e-8))

    def test_l1norm_3(self):
        """Test function for l1norm() 3/4"""
        self.assertTrue(np.allclose(ds.l1norm(self.ABCD_tuple), 2.,
                        rtol=1e-5, atol=1e-8))

    def test_l1norm_4(self):
        """Test function for l1norm() 4/4"""
        self.assertTrue(np.allclose(ds.l1norm(self.splti), 2.,
                        rtol=1e-5, atol=1e-8))




from __future__ import division
import numpy as np

from . import evalTF
from ._utils import _get_zpk

def realizeQNTF(ntf, form='FB', rot=False, bn=None):
    """Convert a quadrature NTF into an ABCD matrix

    The basic idea is to equate the value of the loop filter at a set of
    points in the z-plane to the values of :math:`L_1 = 1-1/H` at those points.

    The order of the zeros is used when mapping the NTF onto the chosen topology.

    Supported structures are:

    * 'FB': Feedback
    * 'PFB': Parallel feedback
    * 'FF': Feedforward (bn is the coefficient of the aux DAC)
    * 'PFF': Parallel feedforward

    Not currently supported - feel free to send in a patch:

    * 'FBD'    FB with delaying quantizer  NOT SUPPORTED YET
    * 'FFD'    FF with delaying quantizer  NOT SUPPORTED YET

    **Parameters:**

    ntf : lti object or supported description
        The NTF to be converted.
    form : str
        The form to be used. See above for the currently supported
        topologies.
    rot : bool, optional
        Set ``rot`` to ``True`` to rotate the states to make as
        many coefficients as possible real. 
    bn : float, optional
        Coefficient of the auxiliary DAC, to be specified for a
        'PFF' form.

    """
    #Code common to all forms
    ntf_z, ntf_p, _ = _get_zpk(ntf)
    order = max(ntf_p.shape)
    A = np.diag(ntf_z)
    A[1:order + 1:A.shape[0]] = 1
    if form == 'PFB' or form == 'PFF':
        # partition the NTF zeros into in-band and image-band
        fz = np.angle(ntf_z)/(2*np.pi)
        f0 = fz[0]
        inband_zeros = abs(fz - f0) < abs(fz + f0)
        n_in = np.sum(inband_zeros)
        imageband_zeros = not inband_zeros
        n_im = np.sum(imageband_zeros)
        if any(imageband_zeros[(n_in + 1 -1):imageband_zeros.shape[0]] != 1):
            raise ValueError('Please put the image-band zeros at the end of' +
                             'the ntf zeros')
        if n_im > 0:
            A[(n_in + 1 -1), (n_in -1)] = 0
    D = np.zeros(shape=(1, 2), dtype='float64')
    # Find a set of points in the z-plane that are not close to zeros of H
    zSet = np.zeros(shape=(np.dot(2, order), 1), dtype='float64')
    for i in range(1, (np.dot(order, 2) +1)):
        z = np.dot(2, rand(1, 1)) - 1 + np.dot(1j, (np.dot(2, rand(1, 1)) - 1))
        while any(abs(z - np.array([ntf_z, zSet]).reshape(1, -1)) < 0.1):
            z = np.dot(2, rand(1, 1)) - 1 + np.dot(1j, (np.dot(2, rand(1, 1)) - 1))
        zSet[(i -1)] = z
    # Evaluate L1 = 1-1/H at these points
    L1 = 1 - 1.0/evalTF(ntf, zSet)
    if 'FB' == form:
        B = np.zeros(shape=(order, 2), dtype='float64')
        C = np.array([np.zeros(shape=(1, order - 1), dtype='float64'), 1]).reshape(1, -1)
        # Compute F = C * inv(zI-A) at each z in zSet
        F = np.zeros(shape=(max(zSet.shape), order), dtype='float64')
        I = eye(order)
        for i in range(1, (max(zSet.shape) +1)):
            F[(i -1), :] = np.dot(C, inv(np.dot(zSet[(i -1)], I) - A))
        B[:, 1] = np.linalg.solve(F, L1)
        if rot == 1:
            ABCD = np.array([A, B[:, 1], C, 0]).reshape(1, -1)
            for i in range(1, (order +1)):
                phi = angle(ABCD[(i -1), -1])
                ABCD[(i -1), :] = np.dot(ABCD[(i -1), :], exp(np.dot(- 1j, phi)))
                ABCD[:, (i -1)] = np.dot(ABCD[:, (i -1)], exp(np.dot(1j, phi)))
            A, B2, C, D2 = partitionABCD(ABCD) # nargout=4
            B[:, 1] = B2
        B[0, 0] = abs(B[0, 1])
    elif 'PFB' == form:
        B = np.zeros(shape=(order, 2), dtype='float64')
        C = np.array([np.zeros(shape=(1, n_in - 1), dtype='float64'), 1, np.zeros(shape=(1, n_im - 1), dtype='float64'), 1]).reshape(1, -1)
        # Compute F = C * inv(zI-A) at each z in zSet
        F = np.zeros(shape=(max(zSet.shape), order), dtype='float64')
        I = eye(order)
        for i in range(1, (max(zSet.shape) +1)):
            F[(i -1), :] = np.dot(C, inv(np.dot(zSet[(i -1)], I) - A))
        B[:, 1] = np.linalg.solve(F, L1)
        if rot == 1:
            ABCD = np.array([A, B[:, 1], C, 0]).reshape(1, -1)
            for i in range(1, (order +1)):
                phi = angle(ABCD[(i -1), -1])
                ABCD[(i -1), :] = np.dot(ABCD[(i -1), :], exp(np.dot(- 1j, phi)))
                ABCD[:, (i -1)] = np.dot(ABCD[:, (i -1)], exp(np.dot(1j, phi)))
            A, B2, C, D2 = partitionABCD(ABCD) # nargout=4
            B[:, 1] = B2
        B[0, 0] = abs(B[0, 1])
    elif 'FF' == form:
        B0 = np.array([1, np.zeros(shape=(order - 1, 1), dtype='float64')]).reshape(1, -1)
        B = B0
        B[-1] = bn
        # Compute F = inv(zI-A)*B at each z in zSet
        F = np.zeros(shape=(order, max(zSet.shape)), dtype='float64')
        I = eye(order)
        for i in range(1, (max(zSet.shape) +1)):
            F[:, (i -1)] = np.dot(inv(np.dot(zSet[(i -1)], I) - A), B)
        C = L1.T / F
        if rot == 1:
            ABCD = np.array([A, B, C, 0]).reshape(1, -1)
            for i in range(2, (order - 1 +1)):
                phi = angle(ABCD[-1, (i -1)])
                ABCD[(i -1), :] = np.dot(ABCD[(i -1), :], exp(np.dot(1j, phi)))
                ABCD[:, (i -1)] = np.dot(ABCD[:, (i -1)], exp(np.dot(- 1j, phi)))
            A, B, C, D2 = partitionABCD(ABCD) # nargout=4
        B = np.array([- B0, B]).reshape(1, -1)
    elif 'PFF' == form:
        B0 = np.array([1, np.zeros(shape=(order - 1, 1), dtype='float64')]).reshape(1, -1)
        B = B0
        B[(n_in + 1 -1)] = 1
        # Compute F = inv(zI-A)*B at each z in zSet
        F = np.zeros(shape=(order, max(zSet.shape)), dtype='float64')
        I = eye(order)
        for i in range(1, (max(zSet.shape) +1)):
            F[:, (i -1)] = np.dot(inv(np.dot(zSet[(i -1)], I) - A), B)
        C = L1.T / F
        if rot == 1:
            ABCD = np.array([A, B, C, 0]).reshape(1, -1)
            for i in range(2, (order - 1 +1)):
                phi = angle(ABCD[-1, (i -1)])
                ABCD[(i -1), :] = np.dot(ABCD[(i -1), :], exp(np.dot(1j, phi)))
                ABCD[:, (i -1)] = np.dot(ABCD[:, (i -1)], exp(np.dot(- 1j, phi)))
            A, B, C, D2 = partitionABCD(ABCD) # nargout=4
        B = np.array([B0, B]).reshape(1, -1)
    else:
        raise ValueError('Sorry, form %s is not supported.' % form)
    ABCD = np.hstack((np.vstack((A, B)), np.vstack((C, D))))
    return ABCD



# -*- coding: utf-8 -*-
# _db.py
# This module provides the db function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the db() function, used to convert an RMS voltage,
expressed in Volt, or a power, expressed in Watt, to dBm.
"""

from __future__ import division

from warnings import warn

import numpy as np

from ._dbp import dbp
from ._dbv import dbv


def db(x, input_type='voltage', R=1.):
    """Calculate the dB equivalent of the RMS signal ``x``.

    For input type ``"voltage"``, the return value is defined as

    .. math::

        P_{dB} = 20 \\mathrm{log}_{10}\\left(\\frac{x}{R}\\right)


    Otherwise, for input type ``"power"``,

    .. math::

        P_{dB} = 10 \\mathrm{log}_{10}(x)


    **Parameters:**

    x : scalar or sequence
        The signal to be converted.

    input_type : string, optional
        The input type, either ``"voltage"`` or ``"power"``.

    R : float, optional
        The normalization resistor value, used only for voltage inputs.


    **Returns:**

    PdB : scalar or sequence
        The input expressed in dB.


    .. note:: MATLAB provides a function with this exact signature.

    .. seealso:: :func:`undbm`, :func:`undbv`, :func:`undbp`, :func:`dbv`, :func:`dbp`, :func:`dbv`

    """
    if input_type.lower().strip() == 'voltage':
        y = dbv(x) - 10. * np.log10(R)
    elif input_type.lower().strip() == 'power':
        y = dbp(x)
        if R != 1.:
            warn("db called with a non default R value, " +
                 "but R is going to be ignored since input_type is power",
                 RuntimeWarning)
    else:
        raise ValueError("db got input_type %s, instead of voltage or power" % input_type)
    return y



# -*- coding: utf-8 -*-
# _mod2.py
# Module providing the mod2 function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the mod2() utility function
"""

import numpy as np

from ._calculateTF import calculateTF


def mod2():
    """A description of the second-order modulator.

    **Returns:**

    ABCD, NTF, STF : a tuple of (ndarray, lti, lti)
        The elements are the ABCD matrix (ndarray),
        the NTF (lti object), the STF (lti object).

    """
    A = np.array([[1., 0.], [1., 1.]])
    B = np.array([[1., -1.], [1., -2.]])
    C = np.array([[0., 1.]])
    D = np.array([[0., 0.]])
    ABCD = np.vstack((np.hstack((A, B)), np.hstack((C, D))))
    H, G = calculateTF(ABCD)
    return ABCD, H, G



# -*- coding: utf-8 -*-
# _plotPZ.py
# Module providing the plotPZ function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing plotPZ(), useful to plot the poles and zeros of a transfer function.
"""

import numpy as np
import pylab as plt

from ._utils import _get_zpk


def plotPZ(H, color='b', markersize=5, showlist=False):
    """Plot the poles and zeros of a transfer function.

    **Parameters:**

    H : transfer function
        Any supported transfer function representation, 
        eg num/den, zpk, lti...

    color : Any matplotlib-compatible color descr, optional
        For example, 'r' for 'red' or '#000080' for 'navy'.
        You can also specify separately poles and zeros, in a tuple.

    markersize : scalar, optional
        The markers size in points.

    showlist : boolean, optional
        Superimpose a list of the poles and zeros on the plot.

    .. plot::

       import pylab as plt
       from deltasigma import synthesizeNTF, plotPZ
       order = 5
       osr = 32
       f0 = 0.
       Hinf = 1.5
       ntf = synthesizeNTF(order, osr, 2, Hinf, f0)
       plt.figure(figsize=(8, 6))
       plotPZ(ntf, color=('r', 'b'), showlist=True)
       plt.title("NTF singularities")
       plt.show()

    """

    # Parts of the code come from 'pydsm'
    #
    # Original copyright notices:
    #
    # For pydsm
    # Copyright (c) 2012, Sergio Callegari
    # All rights reserved.
    #
    # For Richard Schreier's Delta Sigma toolbox
    # Copyright (c) 2009, Richard Schreier
    
    z, p, _ = _get_zpk(H)

    pole_fmt = {'marker': 'x', 'markersize': markersize}
    zero_fmt = {'marker': 'o', 'markersize': markersize}

    if isinstance(color, list) or isinstance(color, tuple):
        pole_fmt['color'] = color[0]
        zero_fmt['color'] = color[1]
    else:
        pole_fmt['color'] = color
        zero_fmt['color'] = color

    hold_status = plt.ishold()
    plt.grid(True)

    # Plot x and o for poles and zeros, respectively
    plt.plot(p.real, p.imag, linestyle='None', **pole_fmt)
    plt.hold(True)
    if len(z) > 0:
        plt.plot(z.real, z.imag, linestyle='None', **zero_fmt)

    # Draw unit circle, real axis and imag axis
    circle = np.exp(2j*np.pi*np.linspace(0, 1, 100))
    plt.plot(circle.real, circle.imag)
    
    ax = plt.gca()
    ax.set_autoscale_on(False)
    if showlist:
        ax = plt.gca()
        x1, x2, y1, y2 = ax.axis()
        x2 = np.round((x2 - x1)*1.48 + x1, 1)
        ax.axis((x1, x2, y1, y2))
        markers = [] 
        descr = []
        ps = p[p.imag >= 0]
        for pi in ps:
            markers += [plt.Line2D((), (), linestyle='None', **pole_fmt)]
            if pi.imag == 0:
                descr += ['%+.4f' % pi.real]
            else:
                descr += ['%+.4f+/-j%.4f' %  (pi.real, pi.imag)]
        if len(z) > 0:
            for zi in z[z.imag >= 0]:
                markers += [plt.Line2D((), (), linestyle='None', **zero_fmt)]
                if zi.imag == 0:
                    descr += ['%+.4f' % zi.real]
                else:
                    descr += ['%+.4f +/-j%.4f' % (zi.real, zi.imag)]
        plt.legend(markers, descr, title="Poles (x) and zeros (o)", ncol=1, loc='best', 
                   handlelength=.55, prop={'size':10})
    else:
        plt.xlim((-1.1, 1.1))
        plt.ylim((-1.1, 1.1))
    plt.gca().set_aspect('equal')
    
    # plt.axes().set_aspect('equal', 'datalim')
    plt.ylabel('Imag')
    plt.xlabel('Real')

    if not hold_status:
        plt.hold(False)



# -*- coding: utf-8 -*-
# _synthesizeNTF0.py
# Module providing the synthesizeNTF function
# Copyright 2013 Giuseppe Venturini
# This file is distributed with python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.
#
# The following code has been (slightly) modified from pydsm, its original
# copyright notice follows:
#
# Copyright (c) 2012, Sergio Callegari
# All rights reserved.
#
# The py dsm code was ported from the MATLAB Delta Sigma toolbox
# Copyright (c) 2009, Richard Schreier
#
# The three software follow the same license, known as the 2-clause BSD.
# See the LICENSE file for details.

"""
Synthesize a noise transfer function (NTF) for a delta-sigma modulator without
optimizing the result.
"""

from warnings import warn

import numpy as np

from ._config import itn_limit
from ._ds_optzeros import ds_optzeros
from ._evalTF import evalTF
from ._utils import cplxpair


def synthesizeNTF0(order, osr, opt, H_inf, f0):
    """Synthesize a noise transfer function for a delta-sigma modulator.

    ::warn This function is not meant to be used directly, instead, set
    optimize_NTF to False and call synthesizeNTF(...).

    Parameters
    ----------
    order : int,
        the order of the modulator
    osr : float,
        the oversamping ratio
    opt : int or list of floats
        flag for optimized zeros

        * 0 -> not optimized,
        * 1 -> optimized,
        * 2 -> optimized with at least one zero at band-center,
        * [z] -> zero locations in complex form

    H_inf : real
        max allowed peak value of the NTF.
    f0 : real
        center frequency for BP modulators, or 0 for LP modulators.
        Note that 1 corresponds to the sampling frequency, so that 0.5 is the
        maximum value. Value 0 specifies an LP modulator.

    Returns
    -------
    ntf : tuple
        noise transfer function in zpk form.

    Raises
    ------
    ValueError
        'Cannot synthesize NTF zeros' if an empty list is supplied for the zeros.

        'Order must be even for a bandpass modulator.' if the order is
        incompatible with the modulator type.

        'The opt vector must be of length xxx' if opt is used to explicitly
        pass the NTF zeros and these are in the wrong number.

    Warns
    -----
        'Unable to achieve specified H_inf ...' if the desired H_inf
        cannot be achieved.

        'Iteration limit exceeded' if the routine converges too slowly.

    Notes
    -----
    This is actually a wrapper function which calls the appropriate version
    of synthesizeNTF, based on the module control flag `optimize_NTF` which
    determines whether to use optimization tools.

    Parameter H_inf is used to enforce the Lee stability criterion.

    See also:
       clans()   "Closed-loop analysis of noise-shaper." An alternative
                 method for selecting NTFs based on the 1-norm of the
                 impulse response of the NTF

       synthesizeChebyshevNTF()    Select a type-2 highpass Chebyshev NTF.
                 This function does a better job than synthesizeNTF if osr
                 or H_inf is low.
    """

    # Determine the zeros.
    if f0 != 0:
        # Bandpass design-- halve the order temporarily.
        order = order/2
        dw = np.pi/(2*osr)
    else:
        dw = np.pi/osr

    if np.isscalar(opt):
        # opt is a number
        if opt == 0:
            z = np.zeros(order)
        else:
            z = dw*ds_optzeros(order, opt)
        if z.size == 0:
            raise ValueError('Cannot synthesize NTF zeros')
        if f0 != 0:
            # Bandpass design-- shift and replicate the zeros.
            order = order*2
            z = z + 2*np.pi*f0
            z = np.vstack((z, -z)).transpose().flatten()
        z = np.exp(1j*z)
    else:
        z = opt

    p = np.zeros(order)
    k = 1
    fprev = 0

    if f0 == 0:
        # Lowpass design
        HinfLimit = 2**order
        # !!! The limit is actually lower for opt=1 and low OSR
        if H_inf >= HinfLimit:
            warn('Unable to achieve specified H_inf.\n'
                 'Setting all NTF poles to zero.')
            p = np.zeros(order)
        else:
            x = 0.3**(order-1)   # starting guess
            for itn in range(1, itn_limit + 1):
                me2 = -0.5*(x**(2./order))
                w = (2*np.arange(1, order + 1) + 1)*np.pi/order
                mb2 = 1 + me2*np.exp(1j*w)
                p = mb2 - np.sqrt(mb2**2 - 1)
                # Reflect poles to be inside the unit circle
                out = abs(p) > 1
                p[out] = 1./p[out]
                # The following is not exactly what delsig does.
                # We do not have an identical cplxpair
                p = cplxpair(p)
                f = np.real(evalTF((z, p, k), -1))-H_inf
                if itn == 1:
                    delta_x = -f/100.
                else:
                    delta_x = -f*delta_x/(f - fprev)
                xplus = x + delta_x
                if xplus > 0:
                    x = xplus
                else:
                    x = x*0.1
                fprev = f
                if abs(f) < 1e-10 or abs(delta_x) < 1e-10:
                    break
                if x > 1e6:
                    warn('Unable to achieve specified Hinf.\n'
                         'Setting all NTF poles to zero.')
                    p = np.zeros(order)
                    break
                if itn == itn_limit:
                    warn('Iteration limit exceeded.')
    else:
        # Bandpass design
        x = 0.3**(order/2-1)   # starting guess (not very good for f0~0)
        if f0 > 0.25:
            z_inf = 1.
        else:
            z_inf = -1.
        c2pif0 = np.cos(2*np.pi*f0)
        for itn in range(1, itn_limit+1):
            e2 = 0.5*x**(2./order)
            w = (2*np.arange(order)+1)*np.pi/order
            mb2 = c2pif0 + e2*np.exp(1j*w)
            p = mb2 - np.sqrt(mb2**2-1)
            # Reflect poles to be inside the unit circle
            out = abs(p)>1
            p[out] = 1/p[out]
            # The following is not exactly what delsig does.
            p = cplxpair(p)
            f = np.real(evalTF((z, p, k), z_inf)) - H_inf
            if itn == 1:
                delta_x = -f/100
            else:
                delta_x = -f*delta_x/(f - fprev)
            xplus = x + delta_x
            if xplus > 0:
                x = xplus
            else:
                x = x*0.1
            fprev = f
            if abs(f) < 1e-10 or abs(delta_x) < 1e-10:
                break
            if x > 1e6:
                warn('Unable to achieve specified Hinf.\n'
                     'Setting all NTF poles to zero.')
                p = np.zeros(order)
                break
            if itn == itn_limit:
                warn('Iteration limit exceeded.')

    z = cplxpair(z)
    return z, p, k



# -*- coding: utf-8 -*-
# test_ds_f1f2.py
# This module provides the tests for the ds_f1f2 function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the ds_f1f2() function.
"""

import unittest
import deltasigma as ds

class TestDs_f1f2(unittest.TestCase):
    """Test class for ds_f1f2()"""
    def setUp(self):
        self.f0 = 1e3
        self.OSR = 128
        self.cf = False

    def test_ds_f1f2_1(self):
        """Test function for ds_f1f2() 1/3"""
        t1f1, t1f2 = ds.ds_f1f2(self.OSR, self.f0, self.cf)
        self.assertTrue((t1f1, t1f2) == (self.f0 - 0.25/self.OSR, self.f0 + 0.25/self.OSR))

    def test_ds_f1f2_2(self):
        """Test function for ds_f1f2() 2/3"""
        t2f1, t2f2 = ds.ds_f1f2(self.OSR, self.f0, not self.cf)
        self.assertTrue((t2f1, t2f2) == (self.f0 - 0.5/self.OSR, self.f0 + 0.5/self.OSR))

    def test_ds_f1f2_3(self):
        """Test function for ds_f1f2() 3/3"""
        t3f1, t3f2 = ds.ds_f1f2(self.OSR, .24/self.OSR, self.cf)
        self.assertTrue((t3f1, t3f2) == (0., 0.5/self.OSR))



# -*- coding: utf-8 -*-
# _synthesizeChebyshevNTF.py
# Module providing the synthesizeChebyshevNTF function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the synthesizeChebyshevNTF() function
"""

from __future__ import division

from warnings import warn

import numpy as np
from scipy.signal import cheby2

from ._ds_f1f2 import ds_f1f2


def synthesizeChebyshevNTF(order=3, OSR=64, opt=0, H_inf=1.5, f0=0.):
    """Synthesize a noise transfer function for a delta-sigma modulator.

    The NTF is a type-2 highpass Chebyshev function.

    :func:`synthesizeNTF` assumes that magnitude of the denominator of the NTF
    is approximately constant in the passband. When the OSR or ``H_inf`` are
    low, this assumption breaks down and :func:`synthesizeNTF` yields a
    non-optimal NTF. :func:`synthesizeChebyshevNTF` creates non-optimal NTFs, but fares
    better than :func:`synthesizeNTF` in the aforementioned circumstances.

    **Parameters:**

    order : int, optional
        order of the modulator, defaults to 3

    OSR : int, optional
        oversampling ratio, defaults to 64

    opt : int, optional
        ignored value, for consistency with :func:`synthesizeNTF`

    H_inf : float, optional
        maximum NTF gain, defaults to 1.5

    f0 : float, optional
        center frequency (1->fs), defaults to 0.

    **Returns:**

    z, p, k : tuple
        a zpk tuple containing the zeros and poles of the NTF.

    **Warns:**

    * If a non-zero value is passed for ``opt``.

    **Raises:**

    * ValueError: Order must be even for a bandpass modulator.

    **Example:**

    Compare the NTFs created by :func:`synthesizeNTF` and
    :func:`synthesizeChebyshevNTF` when ``OSR`` is low::

        OSR = 4
        order = 8
        H_inf = 3
        H0 = synthesizeNTF(order, OSR, 1, H_inf)
        H1 = synthesizeChebyshevNTF(order, OSR, 0, H_inf)

    .. plot::

        import pylab as plt
        import numpy as np
        from deltasigma import *
        OSR = 4
        order = 8
        H_inf = 3
        H0 = synthesizeNTF(order, OSR, 1, H_inf)
        H1 = synthesizeChebyshevNTF(order, OSR, 0, H_inf)
        # 1. Plot the singularities.
        plt.subplot(121)
        # we plot the singularities of the optimized NTF in light
        # green with slightly bigger markers so that we can better
        # distinguish the two NTF's when overlayed.
        plotPZ(H1, markersize=7, color='#90EE90')
        plt.hold(True)
        plotPZ(H0, markersize=5)
        plt.title('NTF Poles and Zeros')
        f = np.concatenate((np.linspace(0, 0.75/OSR, 100), np.linspace(0.75/OSR, 0.5, 100)))
        z = np.exp(2j*np.pi*f)
        magH0 = dbv(evalTF(H0, z))
        magH1 = dbv(evalTF(H1, z))
        # 2. Plot the magnitude responses.
        plt.subplot(222)
        plt.plot(f, magH0, label='synthesizeNTF')
        plt.hold(True)
        plt.plot(f, magH1, label='synthesizeChebyshevNTF')
        figureMagic([0, 0.5], 0.05, None, [-80, 20], 10, None)
        plt.xlabel('Normalized frequency ($1\\\\rightarrow f_s)$')
        plt.ylabel('dB')
        plt.legend(loc=4)
        plt.title('NTF Magnitude Response')
        # 3. Plot the magnitude responses in the signal band.
        plt.subplot(224)
        fstart = 0.01
        f = np.linspace(fstart, 1.2, 200)/(2*OSR)
        z = np.exp(2j*np.pi*f)
        magH0 = dbv(evalTF(H0, z))
        magH1 = dbv(evalTF(H1, z))
        plt.semilogx(f*2*OSR, magH0, label='synthesizeNTF')
        plt.hold(True)
        plt.semilogx(f*2*OSR, magH1, label='synthesizeChebyshevNTF')
        plt.axis([fstart, 1, -50, 0])
        plt.grid(True)
        sigma_H0 = dbv(rmsGain(H0, 0, 0.5/OSR))
        sigma_H1 = dbv(rmsGain(H1, 0, 0.5/OSR))
        plt.semilogx([fstart, 1], sigma_H0*np.array([1, 1]), linewidth=3, color='#191970')
        plt.text(0.15, sigma_H0 + 1.5, 'RMS gain = %5.0fdB' % sigma_H0)
        plt.semilogx([fstart, 1], sigma_H1*np.array([1, 1]), linewidth=3, color='#228B22')
        plt.text(0.15, sigma_H1 + 1.5, 'RMS gain = %5.0fdB' % sigma_H1)
        plt.xlabel('Normalized frequency ($1\\\\rightarrow f_B$)')
        plt.ylabel('dB')
        plt.legend(loc=3)
        plt.tight_layout()

    Repeat for ``H_inf`` low::

        OSR = 32
        order = 5
        H_inf = 1.2
        H0 = synthesizeNTF(order, OSR, 1, H_inf)
        H1 = synthesizeChebyshevNTF(order, OSR, 0, H_inf)

    .. plot::

        import pylab as plt
        import numpy as np
        from deltasigma import *
        OSR = 32
        order = 5
        H_inf = 1.2
        H0 = synthesizeNTF(order, OSR, 1, H_inf)
        H1 = synthesizeChebyshevNTF(order, OSR, 0, H_inf)
        # 1. Plot the singularities.
        plt.subplot(121)
        # we plot the singularities of the optimized NTF in light
        # green with slightly bigger markers so that we can better
        # distinguish the two NTF's when overlayed.
        plotPZ(H1, markersize=7, color='#90EE90')
        plt.hold(True)
        plotPZ(H0, markersize=5)
        plt.title('NTF Poles and Zeros')
        f = np.concatenate((np.linspace(0, 0.75/OSR, 100), np.linspace(0.75/OSR, 0.5, 100)))
        z = np.exp(2j*np.pi*f)
        magH0 = dbv(evalTF(H0, z))
        magH1 = dbv(evalTF(H1, z))
        # 2. Plot the magnitude responses.
        plt.subplot(222)
        plt.plot(f, magH0, label='synthesizeNTF')
        plt.hold(True)
        plt.plot(f, magH1, label='synthesizeChebyshevNTF')
        figureMagic([0, 0.5], 0.05, None, [-80, 20], 10, None)
        plt.xlabel('Normalized frequency ($1\\\\rightarrow f_s)$')
        plt.ylabel('dB')
        plt.legend(loc=4)
        plt.title('NTF Magnitude Response')
        # 3. Plot the magnitude responses in the signal band.
        plt.subplot(224)
        fstart = 0.01
        f = np.linspace(fstart, 1.2, 200)/(2*OSR)
        z = np.exp(2j*np.pi*f)
        magH0 = dbv(evalTF(H0, z))
        magH1 = dbv(evalTF(H1, z))
        plt.semilogx(f*2*OSR, magH0, label='synthesizeNTF')
        plt.hold(True)
        plt.semilogx(f*2*OSR, magH1, label='synthesizeChebyshevNTF')
        plt.axis([fstart, 1, -60, -20])
        plt.grid(True)
        sigma_H0 = dbv(rmsGain(H0, 0, 0.5/OSR))
        sigma_H1 = dbv(rmsGain(H1, 0, 0.5/OSR))
        plt.semilogx([fstart, 1], sigma_H0*np.array([1, 1]), linewidth=3, color='#191970')
        plt.text(0.15, sigma_H0 + 1.5, 'RMS gain = %5.0fdB' % sigma_H0)
        plt.semilogx([fstart, 1], sigma_H1*np.array([1, 1]), linewidth=3, color='#228B22')
        plt.text(0.15, sigma_H1 + 1.5, 'RMS gain = %5.0fdB' % sigma_H1)
        plt.xlabel('Normalized frequency ($1\\\\rightarrow f_B$)')
        plt.ylabel('dB')
        plt.legend(loc=3)
        plt.tight_layout()

    """
    if opt:
        warn("Got a non-zero 'opt' value. Not such optimization is " + \
             "available, opt is only meant to ease switching between " + \
             "synthesizeNTF and synthesizeChebyshevNTF.")
    if f0 != 0:
        if order % 2 != 0:
            raise ValueError('Order must be even for a bandpass modulator.')
        else:
            f1, f2 = ds_f1f2(OSR, f0)
            f1f2 = np.array([f1, f2])
    x_min = 0
    x_max = 300
    dx_max = 10
    ftol = 1e-06
    xtol = 1e-06
    x = 60
    f_p = None # will be redefined later
    itn_limit = 10
    converged = False
    for itn in range(itn_limit):
        if f0 == 0:
            z, p, k = cheby2(order, x, 1./OSR, btype='high', output='zpk')
        else:
            z, p, k = cheby2(order/2., x, 2.*f1f2, btype='stop', output='zpk')
        f = 1./k - H_inf
        if f > 0:
            x_max = x
        else:
            x_min = x
        if itn == 0:
            dx = -dx_max*np.sign(f)
        else:
            df = f - f_p
            if abs(df) < ftol:
                converged = True
                break
            dx = -f*dx/df
        x_p = x
        f_p = f
        x = max(x_min, min(x + dx, x_max))
        dx = x - x_p
        if abs(dx) < xtol:
            break
    ntf = (z, p, 1)
    return ntf



# -*- coding: utf-8 -*-
# test_figureMagic.py
# This module provides the tests for the figureMagic function.
# Copyright 2014 Giuseppe Venturini & Shayne Hodge
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

import unittest
import numpy as np
import deltasigma as ds
import pylab as plt


class TestFigureMagic(unittest.TestCase):
    """Test functions for figureMagic()"""

    def setUp(self):
        pass

    def test_figureMagic(self):
        """test plotting - None should be returned."""
        a = np.arange(10)
        plt.figure()
        plt.plot(a)
        self.assertIsNone(
            ds.figureMagic(
                xRange=[1, 10], dx=1, xLab=None, yRange=[2, 8], dy=.5,
                yLab=None, size=(10, 6), name="Test plot"))



# -*- coding: utf-8 -*-
# _constants.py
# This module holds a few constants that are built-in in MATLAB and that were
# not found in Python.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module holds a few constants that are built-in in MATLAB and that were
not found in Python.
"""

import numpy as np

eps = np.finfo(np.float).eps # x86 2.22044604925e-16



# -*- coding: utf-8 -*-
# _nabsH.py
# This module provides the nabsH function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the nabsH() function, which computes the negative of
the absolute value of H(z).
"""

import numpy as np

from ._evalTF import evalTF


def nabsH(w, H):
    """Computes the negative of the absolute value of H.

    The computation is performed at the specified angular
    frequency ``w``, on the unit circle.

    This function is used by :func:`infnorm`.
    """
    z = np.exp(1j*w)
    return -np.abs(evalTF(H, z))



# -*- coding: utf-8 -*-
# _ds_freq.py
# This module provides the ds_freq function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the ds_freq() function, used to generate a frequency 
vector suitable for plotting the frequency response of an NTF.
"""

from __future__ import division

import numpy as np


def ds_freq(osr=64., f0=0., quadrature=False):
	"""Frequency vector suitable for plotting the frequency response of an NTF
	"""
	if quadrature:
		f_left = -0.5
		f_special = (f0, -f0)
	else:
		f_left = 0.
		f_special = (f0, )
	f = np.linspace(f_left, 0.5, num=100)
	# Use finer spacing in the vicinity of the passband
	for fx in f_special:
		f1 = max(f_left, fx - 1./osr)
		f2 = min(0.5, fx + 2./osr)
		dels = np.where(np.logical_and(f <= f2, f >= f1))
		f = np.delete(f, dels)
		f = np.sort(np.concatenate((f, np.linspace(f1, f2, num=100))))
	return f



# -*- coding: utf-8 -*-
# test_mod1.py
# This module provides the tests for the mod1 function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the mod1() function.
"""

import unittest
import numpy as np
import deltasigma as ds

class Testmod1(unittest.TestCase):
    """Test class for mod1()"""

    def setUp(self):
        self.ABCDmod1 = [[1., 1., -1.], [1., 0., 0.]]

    def test_mod1(self):
        """Test function for mod1()"""
        ABCD, ntf, stf = ds.mod1()
        self.assertTrue(np.allclose(ABCD, self.ABCDmod1, atol=1e-8, rtol=1e-5))




# -*- coding: utf-8 -*-
# _changeFig.py
# Modify the size of lines, markers and text labels in a plot.
# Supports subplots too.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the changeFig() function
"""

import matplotlib as mpl
import numpy as np
import pylab as plt


def changeFig(fontsize=9, linewidth=1, markersize=6, fig=None):
    """Quickly change several figure parameters.

    Take each axes in the figure, and for each line and text item 
    in the axes, set linewidth, markersize and font size.

    **Parameters:**

    fontsize : scalar, optional
        the font size, given in points

    linewidth : scalar, optional
        the line width, given in points

    markersize : scalar, optional
        the marker size, given in points

    fig : a matplotlib figure object, optional
        the figure to apply the modifications to, if not given it is assumed
        to be a the currently active figure.

    **Returns:**

    None.

    .. note:: 

        This function may be useful to enhance the readibility of
        figures to be used in presentations.

    """
    if fig is None:
        fig = plt.gcf()
    for ax in fig.get_axes():
        _setAxLinewidth(ax, linewidth, markersize)
        _setAxLabelsFontsize(ax, fontsize)
        _setTextFontsize(ax, fontsize)

def _setAxLinewidth(ax, linewidth, markersize, BW=False):
    """Take each Line2D in the axes, ax, and convert the line style
    Optionally also convert to BW, from http://tinyurl.com/qylqgoz
    """
    COLORMAP = {
        'b': {'marker': None, 'dash': (None,None)},
        'g': {'marker': None, 'dash': [5,5]},
        'r': {'marker': None, 'dash': [5,3,1,3]},
        'c': {'marker': None, 'dash': [1,3]},
        'm': {'marker': None, 'dash': [5,2,5,2,5,10]},
        'y': {'marker': None, 'dash': [5,3,1,2,1,10]},
        'k': {'marker': 'o', 'dash': (None,None)} #[1,2,1,10]}
        }

    for line in ax.get_lines():
        if BW:
            origColor = line.get_color()
            line.set_color('black')
            line.set_dashes(COLORMAP[origColor]['dash'])
            line.set_marker(COLORMAP[origColor]['marker'])
        if markersize is not None:
            line.set_markersize(markersize)
        if markersize is not None:
            line.set_linewidth(linewidth)

def _setAxLabelsFontsize(ax, fontsize):
    """Change the font size of the axis labels
    """
    for item in ([ax.title, ax.xaxis.label, ax.yaxis.label] +
                 ax.get_xticklabels() + ax.get_yticklabels()):
        item.set_fontsize(fontsize)

def _setTextFontsize(ax, fontsize):
    """Change the font size of the text Artists
    """
    for artist in ax.get_children():
        if isinstance(artist, mpl.text.Text):
            artist.set_size(fontsize)



# -*- coding: utf-8 -*-
# test_bquantize.py
# Bipolar quantization test module
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This is the test module for bquantize()"""

from __future__ import division, print_function
import unittest
import numpy as np
import deltasigma as ds
import scipy.io
import pkg_resources


class TestBQuantize(unittest.TestCase):
    """Test class for bquantize()"""
    def setUp(self):
        self.ran = np.arange(100)
    
    def test_bquantize(self):
        """Test function for bquantize()
        """
        x = np.linspace(-10, 10, 101)
        y = ds.bquantize(x)
        yval = [yi.val for yi in y]
        ycsd = [yi.csd for yi in y]
        fname = pkg_resources.resource_filename(__name__, "test_data/test_bquantize.mat")
        s = scipy.io.loadmat(fname)['s']
        mval = []
        mcsd = []
        for i in range(s.shape[1]):
            mval.append(float(s[0, i][0]))
            mcsd.append(s[0, i][1])
        for i in range(len(mval)):
            self.assertTrue(np.allclose(mval[i], yval[i], atol=1e-8, rtol=1e-5))
            self.assertTrue(np.prod(mcsd[i].shape) + np.prod(ycsd[i].shape) == 0 or \
                   mcsd[i].shape == ycsd[i].shape)
            if 0 not in ycsd[i].shape:
                self.assertTrue(np.allclose(mcsd[i], ycsd[i], atol=1e-8, rtol=1e-5))




# -*- coding: utf-8 -*-
# test_ds_quantize.py
# This module provides the tests for the ds_quantize() function.
# Copyright 2014 Giuseppe Venturini & Shayne Hodge
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

import unittest
import numpy as np
import deltasigma as ds


class TestDSQuantize(unittest.TestCase):
    """Test class for ds_quantize()"""

    def setUp(self):
        t = np.arange(-3, 3, .2)
        t = t.reshape((1, t.shape[0]))
        y = t
        for _ in range(2):
            y = np.concatenate((y, t), axis=0)
        n = np.array([2, 3, 4])
        self.re1_first = ds.ds_quantize(y, n)
        self.re1_second = ds.ds_quantize(y[1, :], n=3)
        self.re2 = np.array(
                [[-1., -1., -1., -1., -1., -1., -1., -1., -1., -1., -1.,
                  -1., -1., -1., -1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                   1., 1., 1., 1., 1., 1.],
                 [-2., -2., -2., -2., -2., -2., -2., -2., -2., -2., 0., 0.,
                   0., 0., 0., 0., 0., 0., 0., 0., 2., 2., 2., 2., 2., 2.,
                   2., 2., 2., 2.],
                 [-3., -3., -3., -3., -3., -1., -1., -1., -1., -1., -1., -1.,
                  -1., -1., -1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 3.,
                   3., 3., 3., 3.]])

    def test_first_quantize(self):
        """Test function for ds_quantize() 1/2 """
        self.assertTrue(
            np.allclose(self.re1_first, self.re2, atol=1e-8, rtol=1e-5))

    def test_second_quantizer(self):
        """Test function for ds_quantize() 2/2 """
        self.assertTrue(
            np.allclose(self.re1_second, self.re2[1, :], atol=1e-8, rtol=1e-5))
